#region Copyright (C) 2005-2008 Team MediaPortal

/* 
 *	Copyright (C) 2005-2008 Team MediaPortal
 *	http://www.team-mediaportal.com
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  This Program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *  http://www.gnu.org/copyleft/gpl.html
 *
 */

#endregion

using System;
using System.Diagnostics;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Xml;
using System.IO;
using System.Runtime.InteropServices;
using MediaPortal.GUI.Library;
using MediaPortal.Util;
using System.Threading;
using MediaPortal.Configuration;
using System.Reflection;

namespace KeyboardInputPlugin // KBIP CHANGED: namespace
{
    /// <summary>
    /// Summary description for ButtonMappingForm.
    /// </summary>

    public class InputMappingForm : MediaPortal.UserInterface.Controls.MPConfigForm
    {
        const string LOG_PREFIX = "KeyboardInputPlugin: ";

        Array nativeWindowsList = Enum.GetValues(typeof(GUIWindow.Window));
        ArrayList windowsList = new ArrayList();
        ArrayList windowsListFiltered = new ArrayList();

        Array nativeActionList = Enum.GetValues(typeof(MediaPortal.GUI.Library.Action.ActionType));
        ArrayList actionList = new ArrayList();

        string[] layerList = new string[] { "all", "1", "2" };
        string[] fullScreenList = new string[] { "Fullscreen", "No Fullscreen" };

        string[] nativePlayerList = new string[] { "TV", "DVD", "MEDIA" };
        string[] playerList = new string[] { "TV is running", "DVD is playing", "Media is playing" };

        string[] nativePowerList = new string[] { "EXIT", "REBOOT", "SHUTDOWN", "STANDBY", "HIBERNATE" };
        string[] powerList = new string[] { "Exit MediaPortal", "Reboot Windows", "Shutdown Windows", "Standby Windows", "Hibernate Windows" };

        string[] nativeProcessList = new string[] { "CLOSE", "KILL" };
        string[] processList = new string[] { "Close Process", "Kill Process" };

        string[] soundList = new string[] { "none", "back.wav", "click.wav", "cursor.wav" };
        string[] keyList = new string[] {"{BACKSPACE}", "{BREAK}", "{CAPSLOCK}", "{DELETE}", "{DOWN}", "{END}", "{ENTER}", "{ESC}",
                                              "{HELP}", "{HOME}", "{INSERT}", "{LEFT}", "{NUMLOCK}", "{PGDN}", "{PGUP}", "{PRTSC}",
                                              "{RIGHT}", "{SCROLLLOCK}", "{TAB}", "{UP}", "{F1}", "{F2}", "{F3}", "{F4}", "{F5}", "{F6}",
                                              "{F7}", "{F8}", "{F9}", "{F10}", "{F11}", "{F12}", "{F13}", "{F14}", "{F15}", "{F16}",
                                              "{ADD}", "{SUBTRACT}", "{MULTIPLY}", "{DIVIDE}"};

        string inputClassName;
        bool changedSettings = false;

        class Data
        {
            string type;
            object dataValue;
            object parameter;
            bool focus = false;

            public Data(object newType, object newParameter, object newValue)
            {
                if (newValue == null)
                    newValue = string.Empty;
                if (newParameter == null)
                    newParameter = string.Empty;
                type = (string)newType;
                dataValue = newValue;
                parameter = newParameter;
            }

            public Data(object newType, object newParameter, object newValue, bool newFocus)
            {
                if (newValue == null)
                    newValue = string.Empty;
                if (newParameter == null)
                    newParameter = string.Empty;
                type = (string)newType;
                dataValue = newValue;
                parameter = newParameter;
                focus = newFocus;
            }

            public string Type { get { return type; } }
            public object Value { get { return dataValue; } set { dataValue = value; } }
            public object Parameter { get { return parameter; } set { parameter = value; } }
            public bool Focus { get { return focus; } set { focus = value; } }
        }

        #region Controls
        private System.Windows.Forms.TreeView treeMapping;
        private System.Windows.Forms.TreeNode rootNode;
        private MediaPortal.UserInterface.Controls.MPRadioButton radioButtonWindow;
        private MediaPortal.UserInterface.Controls.MPRadioButton radioButtonFullscreen;
        private MediaPortal.UserInterface.Controls.MPRadioButton radioButtonPlaying;
        private MediaPortal.UserInterface.Controls.MPRadioButton radioButtonNoCondition;
        private MediaPortal.UserInterface.Controls.MPComboBox comboBoxCondProperty;
        private MediaPortal.UserInterface.Controls.MPComboBox comboBoxCmdProperty;
        private MediaPortal.UserInterface.Controls.MPGroupBox groupBoxCondition;
        private MediaPortal.UserInterface.Controls.MPRadioButton radioButtonAction;
        private MediaPortal.UserInterface.Controls.MPRadioButton radioButtonActWindow;
        private MediaPortal.UserInterface.Controls.MPRadioButton radioButtonToggle;
        private MediaPortal.UserInterface.Controls.MPRadioButton radioButtonPower;
        private MediaPortal.UserInterface.Controls.MPComboBox comboBoxSound;
        private MediaPortal.UserInterface.Controls.MPLabel labelSound;
        private MediaPortal.UserInterface.Controls.MPGroupBox groupBoxAction;
        private MediaPortal.UserInterface.Controls.MPGradientLabel headerLabel;
        private MediaPortal.UserInterface.Controls.MPButton buttonSave;
        private MediaPortal.UserInterface.Controls.MPButton buttonSaveClose;
        private MediaPortal.UserInterface.Controls.MPButton buttonCancel;
        private MediaPortal.UserInterface.Controls.MPBeveledLine beveledLine1;
        private MediaPortal.UserInterface.Controls.MPLabel labelLayer;
        private MediaPortal.UserInterface.Controls.MPComboBox comboBoxLayer;
        private MediaPortal.UserInterface.Controls.MPButton buttonUp;
        private MediaPortal.UserInterface.Controls.MPButton buttonDown;
        private MediaPortal.UserInterface.Controls.MPButton buttonRemove;
        private MediaPortal.UserInterface.Controls.MPButton buttonReset;
        private MediaPortal.UserInterface.Controls.MPGroupBox groupBoxLayer;
        private MediaPortal.UserInterface.Controls.MPRadioButton radioButtonProcess;
        private MediaPortal.UserInterface.Controls.MPLabel label1;
        private MediaPortal.UserInterface.Controls.MPTextBox textBoxKeyChar;
        private MediaPortal.UserInterface.Controls.MPTextBox textBoxKeyCode;
        private MediaPortal.UserInterface.Controls.MPLabel labelExpand;
        private MediaPortal.UserInterface.Controls.MPCheckBox checkBoxGainFocus;
        private MediaPortal.UserInterface.Controls.MPButton buttonNew;
        private MediaPortal.UserInterface.Controls.MPGroupBox groupBoxButton;
        private MediaPortal.UserInterface.Controls.MPLabel labelKeyDesc;
        private MediaPortal.UserInterface.Controls.MPLabel labelKeyName;
        private MediaPortal.UserInterface.Controls.MPTextBox textKeyName;
        private MediaPortal.UserInterface.Controls.MPTextBox textKeyDesc;
        #endregion
        private ToolTip tooltip;
        private GroupBox groupBoxMapping;
        private TextBox textBoxMappingName;
        private Label labelMappingName;
        private Button buttonImport;


        private IContainer components;


        public InputMappingForm(string name)
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();

            foreach (GUIWindow.Window wnd in nativeWindowsList)
            {
                if (wnd.ToString().IndexOf("DIALOG") == -1)
                    switch ((int)Enum.Parse(typeof(GUIWindow.Window), wnd.ToString()))
                    {
                        case (int)GUIWindow.Window.WINDOW_ARTIST_INFO:
                        case (int)GUIWindow.Window.WINDOW_DIALOG_DATETIME:
                        case (int)GUIWindow.Window.WINDOW_DIALOG_EXIF:
                        case (int)GUIWindow.Window.WINDOW_DIALOG_FILE:
                        case (int)GUIWindow.Window.WINDOW_DIALOG_FILESTACKING:
                        case (int)GUIWindow.Window.WINDOW_DIALOG_MENU:
                        case (int)GUIWindow.Window.WINDOW_DIALOG_MENU_BOTTOM_RIGHT:
                        case (int)GUIWindow.Window.WINDOW_DIALOG_NOTIFY:
                        case (int)GUIWindow.Window.WINDOW_DIALOG_OK:
                        case (int)GUIWindow.Window.WINDOW_DIALOG_PROGRESS:
                        case (int)GUIWindow.Window.WINDOW_DIALOG_RATING:
                        case (int)GUIWindow.Window.WINDOW_DIALOG_SELECT:
                        case (int)GUIWindow.Window.WINDOW_DIALOG_SELECT2:
                        case (int)GUIWindow.Window.WINDOW_DIALOG_TEXT:
                        case (int)GUIWindow.Window.WINDOW_DIALOG_TVGUIDE:
                        case (int)GUIWindow.Window.WINDOW_DIALOG_YES_NO:
                        case (int)GUIWindow.Window.WINDOW_INVALID:
                        case (int)GUIWindow.Window.WINDOW_MINI_GUIDE:
                        case (int)GUIWindow.Window.WINDOW_TV_CROP_SETTINGS:
                        case (int)GUIWindow.Window.WINDOW_MSNOSD:
                        case (int)GUIWindow.Window.WINDOW_MUSIC:
                        case (int)GUIWindow.Window.WINDOW_MUSIC_COVERART_GRABBER_RESULTS:
                        case (int)GUIWindow.Window.WINDOW_MUSIC_INFO:
                        case (int)GUIWindow.Window.WINDOW_OSD:
                        case (int)GUIWindow.Window.WINDOW_TOPBAR:
                        //case (int)GUIWindow.Window.WINDOW_TOPBARHOME:
                        case (int)GUIWindow.Window.WINDOW_TVMSNOSD:
                        case (int)GUIWindow.Window.WINDOW_TVOSD:
                        case (int)GUIWindow.Window.WINDOW_TVZAPOSD:
                        case (int)GUIWindow.Window.WINDOW_VIDEO_ARTIST_INFO:
                        case (int)GUIWindow.Window.WINDOW_VIDEO_INFO:
                        case (int)GUIWindow.Window.WINDOW_VIRTUAL_KEYBOARD:
                        case (int)GUIWindow.Window.WINDOW_VIRTUAL_WEB_KEYBOARD:
                            break;
                        default:
                            windowsListFiltered.Add(GetFriendlyName(wnd.ToString()));
                            break;
                    }
                windowsList.Add(GetFriendlyName(wnd.ToString()));
                // KBIP ADDED NEXT LINE
                windowIdsToNameMap.Add((int)wnd, wnd.ToString());
            }
            // KBIP ADDED NEXT LINE
            UpdateWindowsForPlugins();


            foreach (MediaPortal.GUI.Library.Action.ActionType actn in nativeActionList)
                actionList.Add(GetFriendlyName(actn.ToString()));

            comboBoxSound.DataSource = soundList;
            comboBoxLayer.DataSource = layerList;
            inputClassName = name;
            InitializeTreeMapping(inputClassName);
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }

            base.Dispose(disposing);
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(InputMappingForm));
            this.treeMapping = new System.Windows.Forms.TreeView();
            this.labelExpand = new MediaPortal.UserInterface.Controls.MPLabel();
            this.buttonReset = new MediaPortal.UserInterface.Controls.MPButton();
            this.buttonRemove = new MediaPortal.UserInterface.Controls.MPButton();
            this.buttonDown = new MediaPortal.UserInterface.Controls.MPButton();
            this.buttonUp = new MediaPortal.UserInterface.Controls.MPButton();
            this.beveledLine1 = new MediaPortal.UserInterface.Controls.MPBeveledLine();
            this.buttonSave = new MediaPortal.UserInterface.Controls.MPButton();
            this.buttonSaveClose = new MediaPortal.UserInterface.Controls.MPButton();
            this.buttonCancel = new MediaPortal.UserInterface.Controls.MPButton();
            this.headerLabel = new MediaPortal.UserInterface.Controls.MPGradientLabel();
            this.groupBoxAction = new MediaPortal.UserInterface.Controls.MPGroupBox();
            this.checkBoxGainFocus = new MediaPortal.UserInterface.Controls.MPCheckBox();
            this.textBoxKeyCode = new MediaPortal.UserInterface.Controls.MPTextBox();
            this.label1 = new MediaPortal.UserInterface.Controls.MPLabel();
            this.textBoxKeyChar = new MediaPortal.UserInterface.Controls.MPTextBox();
            this.radioButtonProcess = new MediaPortal.UserInterface.Controls.MPRadioButton();
            this.labelSound = new MediaPortal.UserInterface.Controls.MPLabel();
            this.comboBoxSound = new MediaPortal.UserInterface.Controls.MPComboBox();
            this.radioButtonAction = new MediaPortal.UserInterface.Controls.MPRadioButton();
            this.radioButtonActWindow = new MediaPortal.UserInterface.Controls.MPRadioButton();
            this.radioButtonToggle = new MediaPortal.UserInterface.Controls.MPRadioButton();
            this.radioButtonPower = new MediaPortal.UserInterface.Controls.MPRadioButton();
            this.comboBoxCmdProperty = new MediaPortal.UserInterface.Controls.MPComboBox();
            this.groupBoxCondition = new MediaPortal.UserInterface.Controls.MPGroupBox();
            this.radioButtonWindow = new MediaPortal.UserInterface.Controls.MPRadioButton();
            this.radioButtonFullscreen = new MediaPortal.UserInterface.Controls.MPRadioButton();
            this.radioButtonPlaying = new MediaPortal.UserInterface.Controls.MPRadioButton();
            this.radioButtonNoCondition = new MediaPortal.UserInterface.Controls.MPRadioButton();
            this.comboBoxCondProperty = new MediaPortal.UserInterface.Controls.MPComboBox();
            this.groupBoxLayer = new MediaPortal.UserInterface.Controls.MPGroupBox();
            this.comboBoxLayer = new MediaPortal.UserInterface.Controls.MPComboBox();
            this.labelLayer = new MediaPortal.UserInterface.Controls.MPLabel();
            this.buttonNew = new MediaPortal.UserInterface.Controls.MPButton();
            this.groupBoxButton = new MediaPortal.UserInterface.Controls.MPGroupBox();
            this.textKeyDesc = new MediaPortal.UserInterface.Controls.MPTextBox();
            this.labelKeyDesc = new MediaPortal.UserInterface.Controls.MPLabel();
            this.labelKeyName = new MediaPortal.UserInterface.Controls.MPLabel();
            this.textKeyName = new MediaPortal.UserInterface.Controls.MPTextBox();
            this.tooltip = new System.Windows.Forms.ToolTip(this.components);
            this.textBoxMappingName = new System.Windows.Forms.TextBox();
            this.buttonImport = new System.Windows.Forms.Button();
            this.groupBoxMapping = new System.Windows.Forms.GroupBox();
            this.labelMappingName = new System.Windows.Forms.Label();
            this.groupBoxAction.SuspendLayout();
            this.groupBoxCondition.SuspendLayout();
            this.groupBoxLayer.SuspendLayout();
            this.groupBoxButton.SuspendLayout();
            this.groupBoxMapping.SuspendLayout();
            this.SuspendLayout();
            // 
            // treeMapping
            // 
            this.treeMapping.AllowDrop = true;
            this.treeMapping.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.treeMapping.FullRowSelect = true;
            this.treeMapping.HideSelection = false;
            this.treeMapping.Location = new System.Drawing.Point(16, 56);
            this.treeMapping.Name = "treeMapping";
            this.treeMapping.Size = new System.Drawing.Size(312, 468);
            this.treeMapping.TabIndex = 1;
            this.tooltip.SetToolTip(this.treeMapping, resources.GetString("treeMapping.ToolTip"));
            this.treeMapping.AfterSelect += new System.Windows.Forms.TreeViewEventHandler(this.treeMapping_AfterSelect);
            // 
            // labelExpand
            // 
            this.labelExpand.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.labelExpand.AutoSize = true;
            this.labelExpand.Location = new System.Drawing.Point(328, 507);
            this.labelExpand.Name = "labelExpand";
            this.labelExpand.Size = new System.Drawing.Size(13, 13);
            this.labelExpand.TabIndex = 29;
            this.labelExpand.Text = "+";
            this.tooltip.SetToolTip(this.labelExpand, "Click here to expand all nodes underneath\r\nthe  currently selected node.");
            this.labelExpand.Click += new System.EventHandler(this.labelExpand_Click);
            // 
            // buttonReset
            // 
            this.buttonReset.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.buttonReset.Location = new System.Drawing.Point(206, 573);
            this.buttonReset.Name = "buttonReset";
            this.buttonReset.Size = new System.Drawing.Size(90, 23);
            this.buttonReset.TabIndex = 28;
            this.buttonReset.Text = "Reset";
            this.tooltip.SetToolTip(this.buttonReset, "Reset the mappings to defaults; this will remove all mappings and,\r\nif <ProgramDi" +
                    "r>\\CustomInputDevices\\defaults\\Keyboard.xml exists,\r\nreload the mappings from th" +
                    "at file.");
            this.buttonReset.UseVisualStyleBackColor = true;
            this.buttonReset.Click += new System.EventHandler(this.buttonReset_Click);
            // 
            // buttonRemove
            // 
            this.buttonRemove.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
            this.buttonRemove.Location = new System.Drawing.Point(272, 530);
            this.buttonRemove.Name = "buttonRemove";
            this.buttonRemove.Size = new System.Drawing.Size(56, 20);
            this.buttonRemove.TabIndex = 27;
            this.buttonRemove.Text = "Remove";
            this.tooltip.SetToolTip(this.buttonRemove, "Remove the currently selected item from the tree");
            this.buttonRemove.UseVisualStyleBackColor = true;
            this.buttonRemove.Click += new System.EventHandler(this.buttonRemove_Click);
            // 
            // buttonDown
            // 
            this.buttonDown.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
            this.buttonDown.Location = new System.Drawing.Point(97, 530);
            this.buttonDown.Name = "buttonDown";
            this.buttonDown.Size = new System.Drawing.Size(56, 20);
            this.buttonDown.TabIndex = 24;
            this.buttonDown.Text = "Down";
            this.tooltip.SetToolTip(this.buttonDown, "Use the Up and Down buttons to re-order items");
            this.buttonDown.UseVisualStyleBackColor = true;
            this.buttonDown.Click += new System.EventHandler(this.buttonDown_Click);
            // 
            // buttonUp
            // 
            this.buttonUp.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
            this.buttonUp.Location = new System.Drawing.Point(16, 530);
            this.buttonUp.Name = "buttonUp";
            this.buttonUp.Size = new System.Drawing.Size(56, 20);
            this.buttonUp.TabIndex = 23;
            this.buttonUp.Text = "Up";
            this.tooltip.SetToolTip(this.buttonUp, "Use the Up and Down buttons to re-order items");
            this.buttonUp.UseVisualStyleBackColor = true;
            this.buttonUp.Click += new System.EventHandler(this.buttonUp_Click);
            // 
            // beveledLine1
            // 
            this.beveledLine1.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.beveledLine1.Location = new System.Drawing.Point(8, 565);
            this.beveledLine1.Name = "beveledLine1";
            this.beveledLine1.Size = new System.Drawing.Size(572, 2);
            this.beveledLine1.TabIndex = 21;
            // 
            // buttonSave
            // 
            this.buttonSave.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.buttonSave.Location = new System.Drawing.Point(302, 573);
            this.buttonSave.Name = "buttonSave";
            this.buttonSave.Size = new System.Drawing.Size(90, 23);
            this.buttonSave.TabIndex = 20;
            this.buttonSave.Text = "Save";
            this.tooltip.SetToolTip(this.buttonSave, "Save any changes, keep the \r\nconfiguration window open");
            this.buttonSave.UseVisualStyleBackColor = true;
            this.buttonSave.Click += new System.EventHandler(this.buttonSave_Click);
            // 
            // buttonSaveClose
            // 
            this.buttonSaveClose.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.buttonSaveClose.Location = new System.Drawing.Point(398, 573);
            this.buttonSaveClose.Name = "buttonSaveClose";
            this.buttonSaveClose.Size = new System.Drawing.Size(90, 23);
            this.buttonSaveClose.TabIndex = 19;
            this.buttonSaveClose.Text = "Save & Close";
            this.tooltip.SetToolTip(this.buttonSaveClose, "Save any changes and close\r\nthe configuration window");
            this.buttonSaveClose.UseMnemonic = false;
            this.buttonSaveClose.UseVisualStyleBackColor = true;
            this.buttonSaveClose.Click += new System.EventHandler(this.buttonSaveClose_Click);
            // 
            // buttonCancel
            // 
            this.buttonCancel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.buttonCancel.Location = new System.Drawing.Point(494, 573);
            this.buttonCancel.Name = "buttonCancel";
            this.buttonCancel.Size = new System.Drawing.Size(90, 23);
            this.buttonCancel.TabIndex = 18;
            this.buttonCancel.Text = "Cancel";
            this.tooltip.SetToolTip(this.buttonCancel, "Close the window without saving\r\nany changes you may have made");
            this.buttonCancel.UseVisualStyleBackColor = true;
            this.buttonCancel.Click += new System.EventHandler(this.buttonCancel_Click);
            // 
            // headerLabel
            // 
            this.headerLabel.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.headerLabel.Caption = "Keyboard Mappings";
            this.headerLabel.FirstColor = System.Drawing.SystemColors.InactiveCaption;
            this.headerLabel.Font = new System.Drawing.Font("Verdana", 14.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.headerLabel.LastColor = System.Drawing.Color.WhiteSmoke;
            this.headerLabel.Location = new System.Drawing.Point(16, 16);
            this.headerLabel.Name = "headerLabel";
            this.headerLabel.PaddingLeft = 2;
            this.headerLabel.Size = new System.Drawing.Size(558, 24);
            this.headerLabel.TabIndex = 17;
            this.headerLabel.TextColor = System.Drawing.Color.WhiteSmoke;
            this.headerLabel.TextFont = new System.Drawing.Font("Verdana", 14.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            // 
            // groupBoxAction
            // 
            this.groupBoxAction.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.groupBoxAction.Controls.Add(this.checkBoxGainFocus);
            this.groupBoxAction.Controls.Add(this.textBoxKeyCode);
            this.groupBoxAction.Controls.Add(this.label1);
            this.groupBoxAction.Controls.Add(this.textBoxKeyChar);
            this.groupBoxAction.Controls.Add(this.radioButtonProcess);
            this.groupBoxAction.Controls.Add(this.labelSound);
            this.groupBoxAction.Controls.Add(this.comboBoxSound);
            this.groupBoxAction.Controls.Add(this.radioButtonAction);
            this.groupBoxAction.Controls.Add(this.radioButtonActWindow);
            this.groupBoxAction.Controls.Add(this.radioButtonToggle);
            this.groupBoxAction.Controls.Add(this.radioButtonPower);
            this.groupBoxAction.Controls.Add(this.comboBoxCmdProperty);
            this.groupBoxAction.Enabled = false;
            this.groupBoxAction.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
            this.groupBoxAction.Location = new System.Drawing.Point(350, 332);
            this.groupBoxAction.Name = "groupBoxAction";
            this.groupBoxAction.Size = new System.Drawing.Size(224, 192);
            this.groupBoxAction.TabIndex = 16;
            this.groupBoxAction.TabStop = false;
            this.groupBoxAction.Text = "Action";
            this.tooltip.SetToolTip(this.groupBoxAction, "Using this section you can define the action to perform\r\nwhen the user presses th" +
                    "e given key combination or button,\r\nif the given condition is true.");
            // 
            // checkBoxGainFocus
            // 
            this.checkBoxGainFocus.AutoSize = true;
            this.checkBoxGainFocus.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
            this.checkBoxGainFocus.Location = new System.Drawing.Point(112, 68);
            this.checkBoxGainFocus.Name = "checkBoxGainFocus";
            this.checkBoxGainFocus.Size = new System.Drawing.Size(78, 17);
            this.checkBoxGainFocus.TabIndex = 25;
            this.checkBoxGainFocus.Text = "Gain Focus";
            this.checkBoxGainFocus.UseVisualStyleBackColor = true;
            this.checkBoxGainFocus.CheckedChanged += new System.EventHandler(this.checkBoxGainFocus_CheckedChanged);
            // 
            // textBoxKeyCode
            // 
            this.textBoxKeyCode.BorderColor = System.Drawing.Color.Empty;
            this.textBoxKeyCode.Enabled = false;
            this.textBoxKeyCode.Location = new System.Drawing.Point(152, 124);
            this.textBoxKeyCode.MaxLength = 3;
            this.textBoxKeyCode.Name = "textBoxKeyCode";
            this.textBoxKeyCode.Size = new System.Drawing.Size(48, 20);
            this.textBoxKeyCode.TabIndex = 24;
            this.textBoxKeyCode.KeyUp += new System.Windows.Forms.KeyEventHandler(this.textBoxKeyCode_KeyUp);
            this.textBoxKeyCode.KeyPress += new System.Windows.Forms.KeyPressEventHandler(this.textBoxKeyCode_KeyPress);
            // 
            // label1
            // 
            this.label1.AutoSize = true;
            this.label1.Location = new System.Drawing.Point(24, 128);
            this.label1.Name = "label1";
            this.label1.Size = new System.Drawing.Size(28, 13);
            this.label1.TabIndex = 23;
            this.label1.Text = "Key:";
            // 
            // textBoxKeyChar
            // 
            this.textBoxKeyChar.BorderColor = System.Drawing.Color.Empty;
            this.textBoxKeyChar.Enabled = false;
            this.textBoxKeyChar.Location = new System.Drawing.Point(72, 124);
            this.textBoxKeyChar.MaxLength = 3;
            this.textBoxKeyChar.Name = "textBoxKeyChar";
            this.textBoxKeyChar.Size = new System.Drawing.Size(80, 20);
            this.textBoxKeyChar.TabIndex = 22;
            this.textBoxKeyChar.KeyUp += new System.Windows.Forms.KeyEventHandler(this.textBoxKeyChar_KeyUp);
            this.textBoxKeyChar.KeyPress += new System.Windows.Forms.KeyPressEventHandler(this.textBoxKeyChar_KeyPress);
            // 
            // radioButtonProcess
            // 
            this.radioButtonProcess.AutoSize = true;
            this.radioButtonProcess.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
            this.radioButtonProcess.Location = new System.Drawing.Point(24, 68);
            this.radioButtonProcess.Name = "radioButtonProcess";
            this.radioButtonProcess.Size = new System.Drawing.Size(62, 17);
            this.radioButtonProcess.TabIndex = 21;
            this.radioButtonProcess.Text = "Process";
            this.radioButtonProcess.UseVisualStyleBackColor = true;
            this.radioButtonProcess.Click += new System.EventHandler(this.radioButtonProcess_Click);
            // 
            // labelSound
            // 
            this.labelSound.AutoSize = true;
            this.labelSound.Location = new System.Drawing.Point(24, 156);
            this.labelSound.Name = "labelSound";
            this.labelSound.Size = new System.Drawing.Size(41, 13);
            this.labelSound.TabIndex = 20;
            this.labelSound.Text = "Sound:";
            // 
            // comboBoxSound
            // 
            this.comboBoxSound.BorderColor = System.Drawing.Color.Empty;
            this.comboBoxSound.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboBoxSound.ForeColor = System.Drawing.Color.DarkRed;
            this.comboBoxSound.Location = new System.Drawing.Point(72, 153);
            this.comboBoxSound.Name = "comboBoxSound";
            this.comboBoxSound.Size = new System.Drawing.Size(128, 21);
            this.comboBoxSound.TabIndex = 19;
            this.comboBoxSound.SelectionChangeCommitted += new System.EventHandler(this.comboBoxSound_SelectionChangeCommitted);
            this.comboBoxSound.SelectedValueChanged += new System.EventHandler(this.comboBoxSound_SelectionChangeCommitted);
            // 
            // radioButtonAction
            // 
            this.radioButtonAction.AutoSize = true;
            this.radioButtonAction.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
            this.radioButtonAction.Location = new System.Drawing.Point(24, 20);
            this.radioButtonAction.Name = "radioButtonAction";
            this.radioButtonAction.Size = new System.Drawing.Size(54, 17);
            this.radioButtonAction.TabIndex = 14;
            this.radioButtonAction.Text = "Action";
            this.radioButtonAction.UseVisualStyleBackColor = true;
            this.radioButtonAction.Click += new System.EventHandler(this.radioButtonAction_Click);
            // 
            // radioButtonActWindow
            // 
            this.radioButtonActWindow.AutoSize = true;
            this.radioButtonActWindow.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
            this.radioButtonActWindow.Location = new System.Drawing.Point(112, 20);
            this.radioButtonActWindow.Name = "radioButtonActWindow";
            this.radioButtonActWindow.Size = new System.Drawing.Size(63, 17);
            this.radioButtonActWindow.TabIndex = 14;
            this.radioButtonActWindow.Text = "Window";
            this.radioButtonActWindow.UseVisualStyleBackColor = true;
            this.radioButtonActWindow.Click += new System.EventHandler(this.radioButtonActWindow_Click);
            // 
            // radioButtonToggle
            // 
            this.radioButtonToggle.AutoSize = true;
            this.radioButtonToggle.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
            this.radioButtonToggle.Location = new System.Drawing.Point(112, 44);
            this.radioButtonToggle.Name = "radioButtonToggle";
            this.radioButtonToggle.Size = new System.Drawing.Size(86, 17);
            this.radioButtonToggle.TabIndex = 17;
            this.radioButtonToggle.Text = "Toggle Layer";
            this.radioButtonToggle.UseVisualStyleBackColor = true;
            this.radioButtonToggle.Click += new System.EventHandler(this.radioButtonToggle_Click);
            // 
            // radioButtonPower
            // 
            this.radioButtonPower.AutoSize = true;
            this.radioButtonPower.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
            this.radioButtonPower.Location = new System.Drawing.Point(24, 44);
            this.radioButtonPower.Name = "radioButtonPower";
            this.radioButtonPower.Size = new System.Drawing.Size(80, 17);
            this.radioButtonPower.TabIndex = 18;
            this.radioButtonPower.Text = "Powerdown";
            this.radioButtonPower.UseVisualStyleBackColor = true;
            this.radioButtonPower.Click += new System.EventHandler(this.radioButtonPower_Click);
            // 
            // comboBoxCmdProperty
            // 
            this.comboBoxCmdProperty.BorderColor = System.Drawing.Color.Empty;
            this.comboBoxCmdProperty.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboBoxCmdProperty.ForeColor = System.Drawing.Color.DarkGreen;
            this.comboBoxCmdProperty.Location = new System.Drawing.Point(24, 92);
            this.comboBoxCmdProperty.Name = "comboBoxCmdProperty";
            this.comboBoxCmdProperty.Size = new System.Drawing.Size(176, 21);
            this.comboBoxCmdProperty.Sorted = true;
            this.comboBoxCmdProperty.TabIndex = 14;
            this.comboBoxCmdProperty.SelectionChangeCommitted += new System.EventHandler(this.comboBoxCmdProperty_SelectionChangeCommitted);
            this.comboBoxCmdProperty.SelectedValueChanged += new System.EventHandler(this.comboBoxCmdProperty_SelectionChangeCommitted);
            // 
            // groupBoxCondition
            // 
            this.groupBoxCondition.Anchor = System.Windows.Forms.AnchorStyles.Right;
            this.groupBoxCondition.Controls.Add(this.radioButtonWindow);
            this.groupBoxCondition.Controls.Add(this.radioButtonFullscreen);
            this.groupBoxCondition.Controls.Add(this.radioButtonPlaying);
            this.groupBoxCondition.Controls.Add(this.radioButtonNoCondition);
            this.groupBoxCondition.Controls.Add(this.comboBoxCondProperty);
            this.groupBoxCondition.Enabled = false;
            this.groupBoxCondition.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
            this.groupBoxCondition.Location = new System.Drawing.Point(350, 235);
            this.groupBoxCondition.Name = "groupBoxCondition";
            this.groupBoxCondition.Size = new System.Drawing.Size(224, 100);
            this.groupBoxCondition.TabIndex = 15;
            this.groupBoxCondition.TabStop = false;
            this.groupBoxCondition.Text = "Condition";
            this.tooltip.SetToolTip(this.groupBoxCondition, "Using this section, you can provide a condition \r\nfor the action to be active.");
            // 
            // radioButtonWindow
            // 
            this.radioButtonWindow.AutoSize = true;
            this.radioButtonWindow.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
            this.radioButtonWindow.Location = new System.Drawing.Point(24, 20);
            this.radioButtonWindow.Name = "radioButtonWindow";
            this.radioButtonWindow.Size = new System.Drawing.Size(63, 17);
            this.radioButtonWindow.TabIndex = 9;
            this.radioButtonWindow.Text = "Window";
            this.radioButtonWindow.UseVisualStyleBackColor = true;
            this.radioButtonWindow.Click += new System.EventHandler(this.radioButtonWindow_Click);
            // 
            // radioButtonFullscreen
            // 
            this.radioButtonFullscreen.AutoSize = true;
            this.radioButtonFullscreen.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
            this.radioButtonFullscreen.Location = new System.Drawing.Point(112, 20);
            this.radioButtonFullscreen.Name = "radioButtonFullscreen";
            this.radioButtonFullscreen.Size = new System.Drawing.Size(72, 17);
            this.radioButtonFullscreen.TabIndex = 10;
            this.radioButtonFullscreen.Text = "Fullscreen";
            this.radioButtonFullscreen.UseVisualStyleBackColor = true;
            this.radioButtonFullscreen.Click += new System.EventHandler(this.radioButtonFullscreen_Click);
            // 
            // radioButtonPlaying
            // 
            this.radioButtonPlaying.AutoSize = true;
            this.radioButtonPlaying.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
            this.radioButtonPlaying.Location = new System.Drawing.Point(24, 44);
            this.radioButtonPlaying.Name = "radioButtonPlaying";
            this.radioButtonPlaying.Size = new System.Drawing.Size(58, 17);
            this.radioButtonPlaying.TabIndex = 11;
            this.radioButtonPlaying.Text = "Playing";
            this.radioButtonPlaying.UseVisualStyleBackColor = true;
            this.radioButtonPlaying.Click += new System.EventHandler(this.radioButtonPlaying_Click);
            // 
            // radioButtonNoCondition
            // 
            this.radioButtonNoCondition.AutoSize = true;
            this.radioButtonNoCondition.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
            this.radioButtonNoCondition.Location = new System.Drawing.Point(112, 44);
            this.radioButtonNoCondition.Name = "radioButtonNoCondition";
            this.radioButtonNoCondition.Size = new System.Drawing.Size(85, 17);
            this.radioButtonNoCondition.TabIndex = 12;
            this.radioButtonNoCondition.Text = "No Condition";
            this.radioButtonNoCondition.UseVisualStyleBackColor = true;
            this.radioButtonNoCondition.Click += new System.EventHandler(this.radioButtonNoCondition_Click);
            // 
            // comboBoxCondProperty
            // 
            this.comboBoxCondProperty.BorderColor = System.Drawing.Color.Empty;
            this.comboBoxCondProperty.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboBoxCondProperty.ForeColor = System.Drawing.Color.Blue;
            this.comboBoxCondProperty.Location = new System.Drawing.Point(24, 68);
            this.comboBoxCondProperty.Name = "comboBoxCondProperty";
            this.comboBoxCondProperty.Size = new System.Drawing.Size(176, 21);
            this.comboBoxCondProperty.Sorted = true;
            this.comboBoxCondProperty.TabIndex = 13;
            this.comboBoxCondProperty.SelectionChangeCommitted += new System.EventHandler(this.comboBoxCondProperty_SelectionChangeCommitted);
            this.comboBoxCondProperty.SelectedValueChanged += new System.EventHandler(this.comboBoxCondProperty_SelectionChangeCommitted);
            // 
            // groupBoxLayer
            // 
            this.groupBoxLayer.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.groupBoxLayer.Controls.Add(this.comboBoxLayer);
            this.groupBoxLayer.Controls.Add(this.labelLayer);
            this.groupBoxLayer.Enabled = false;
            this.groupBoxLayer.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
            this.groupBoxLayer.Location = new System.Drawing.Point(350, 177);
            this.groupBoxLayer.Name = "groupBoxLayer";
            this.groupBoxLayer.Size = new System.Drawing.Size(224, 52);
            this.groupBoxLayer.TabIndex = 22;
            this.groupBoxLayer.TabStop = false;
            this.groupBoxLayer.Text = "Layer";
            this.tooltip.SetToolTip(this.groupBoxLayer, "Using this section, you can select a layer.");
            // 
            // comboBoxLayer
            // 
            this.comboBoxLayer.BorderColor = System.Drawing.Color.Empty;
            this.comboBoxLayer.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboBoxLayer.ForeColor = System.Drawing.Color.DimGray;
            this.comboBoxLayer.Location = new System.Drawing.Point(80, 20);
            this.comboBoxLayer.Name = "comboBoxLayer";
            this.comboBoxLayer.Size = new System.Drawing.Size(121, 21);
            this.comboBoxLayer.TabIndex = 25;
            this.comboBoxLayer.SelectionChangeCommitted += new System.EventHandler(this.comboBoxLayer_SelectionChangeCommitted);
            this.comboBoxLayer.SelectedValueChanged += new System.EventHandler(this.comboBoxLayer_SelectionChangeCommitted);
            // 
            // labelLayer
            // 
            this.labelLayer.AutoSize = true;
            this.labelLayer.Location = new System.Drawing.Point(24, 23);
            this.labelLayer.Name = "labelLayer";
            this.labelLayer.Size = new System.Drawing.Size(36, 13);
            this.labelLayer.TabIndex = 16;
            this.labelLayer.Text = "Layer:";
            // 
            // buttonNew
            // 
            this.buttonNew.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
            this.buttonNew.Location = new System.Drawing.Point(189, 530);
            this.buttonNew.Name = "buttonNew";
            this.buttonNew.Size = new System.Drawing.Size(56, 20);
            this.buttonNew.TabIndex = 26;
            this.buttonNew.Text = "New";
            this.tooltip.SetToolTip(this.buttonNew, "Depending on the selection in the tree list above,\r\nthis button allows you to cre" +
                    "ate a new Mapping\r\ndefinition, a new Key/Button definition, a new layer,\r\nor a n" +
                    "ew condition.");
            this.buttonNew.UseVisualStyleBackColor = true;
            this.buttonNew.Click += new System.EventHandler(this.buttonNew_Click);
            // 
            // groupBoxButton
            // 
            this.groupBoxButton.Controls.Add(this.textKeyDesc);
            this.groupBoxButton.Controls.Add(this.labelKeyDesc);
            this.groupBoxButton.Controls.Add(this.labelKeyName);
            this.groupBoxButton.Controls.Add(this.textKeyName);
            this.groupBoxButton.Enabled = false;
            this.groupBoxButton.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
            this.groupBoxButton.Location = new System.Drawing.Point(350, 99);
            this.groupBoxButton.Name = "groupBoxButton";
            this.groupBoxButton.Size = new System.Drawing.Size(224, 72);
            this.groupBoxButton.TabIndex = 30;
            this.groupBoxButton.TabStop = false;
            this.groupBoxButton.Text = "Key/Button Definition";
            this.tooltip.SetToolTip(this.groupBoxButton, "Using this section, you can define a key or button definition.\r\nA definition cont" +
                    "ains a name that identifies the button or\r\nkey combination, and the actual key o" +
                    "r button.");
            // 
            // textKeyDesc
            // 
            this.textKeyDesc.BorderColor = System.Drawing.Color.Empty;
            this.textKeyDesc.Location = new System.Drawing.Point(48, 43);
            this.textKeyDesc.Name = "textKeyDesc";
            this.textKeyDesc.Size = new System.Drawing.Size(172, 20);
            this.textKeyDesc.TabIndex = 107;
            this.tooltip.SetToolTip(this.textKeyDesc, "Click in this field and press the remote control button\r\nor key combination that " +
                    "you would like to assign to\r\nthis key/button definition.");
            this.textKeyDesc.GotFocus += new System.EventHandler(this.textKeyDesc_GotFocus);
            this.textKeyDesc.LostFocus += new System.EventHandler(this.textKeyDesc_LostFocus);
            // 
            // labelKeyDesc
            // 
            this.labelKeyDesc.AutoSize = true;
            this.labelKeyDesc.Location = new System.Drawing.Point(4, 46);
            this.labelKeyDesc.Name = "labelKeyDesc";
            this.labelKeyDesc.Size = new System.Drawing.Size(28, 13);
            this.labelKeyDesc.TabIndex = 109;
            this.labelKeyDesc.Text = "Key:";
            // 
            // labelKeyName
            // 
            this.labelKeyName.AutoSize = true;
            this.labelKeyName.Location = new System.Drawing.Point(4, 20);
            this.labelKeyName.Name = "labelKeyName";
            this.labelKeyName.Size = new System.Drawing.Size(38, 13);
            this.labelKeyName.TabIndex = 110;
            this.labelKeyName.Text = "Name:";
            // 
            // textKeyName
            // 
            this.textKeyName.BorderColor = System.Drawing.Color.Empty;
            this.textKeyName.Location = new System.Drawing.Point(48, 17);
            this.textKeyName.Name = "textKeyName";
            this.textKeyName.Size = new System.Drawing.Size(172, 20);
            this.textKeyName.TabIndex = 106;
            this.tooltip.SetToolTip(this.textKeyName, "Here you can enter a name that describes\r\nyour button or key combination.");
            this.textKeyName.KeyUp += new System.Windows.Forms.KeyEventHandler(this.textKeyName_KeyUp);
            // 
            // tooltip
            // 
            this.tooltip.AutoPopDelay = 20000;
            this.tooltip.InitialDelay = 400;
            this.tooltip.IsBalloon = true;
            this.tooltip.ReshowDelay = 400;
            // 
            // textBoxMappingName
            // 
            this.textBoxMappingName.Location = new System.Drawing.Point(48, 19);
            this.textBoxMappingName.Name = "textBoxMappingName";
            this.textBoxMappingName.Size = new System.Drawing.Size(172, 20);
            this.textBoxMappingName.TabIndex = 1;
            this.tooltip.SetToolTip(this.textBoxMappingName, "Here you can enter a name that describes your \r\nmapping, for example the name of " +
                    "the remote \r\ncontrol that this mapping applies to.");
            this.textBoxMappingName.KeyUp += new System.Windows.Forms.KeyEventHandler(this.textBoxMappingName_KeyUp);
            // 
            // buttonImport
            // 
            this.buttonImport.Location = new System.Drawing.Point(16, 572);
            this.buttonImport.Name = "buttonImport";
            this.buttonImport.Size = new System.Drawing.Size(75, 23);
            this.buttonImport.TabIndex = 32;
            this.buttonImport.Text = "Import";
            this.tooltip.SetToolTip(this.buttonImport, "Import mapping definitions from a mappings XML file\r\ninto your current mapping. Y" +
                    "ou can use this to load\r\npredefined mappings.");
            this.buttonImport.UseVisualStyleBackColor = true;
            this.buttonImport.Click += new System.EventHandler(this.buttonImport_Click);
            // 
            // groupBoxMapping
            // 
            this.groupBoxMapping.Controls.Add(this.textBoxMappingName);
            this.groupBoxMapping.Controls.Add(this.labelMappingName);
            this.groupBoxMapping.Enabled = false;
            this.groupBoxMapping.Location = new System.Drawing.Point(350, 46);
            this.groupBoxMapping.Name = "groupBoxMapping";
            this.groupBoxMapping.Size = new System.Drawing.Size(224, 47);
            this.groupBoxMapping.TabIndex = 31;
            this.groupBoxMapping.TabStop = false;
            this.groupBoxMapping.Text = "Mapping Definition";
            // 
            // labelMappingName
            // 
            this.labelMappingName.AutoSize = true;
            this.labelMappingName.Location = new System.Drawing.Point(6, 22);
            this.labelMappingName.Name = "labelMappingName";
            this.labelMappingName.Size = new System.Drawing.Size(38, 13);
            this.labelMappingName.TabIndex = 0;
            this.labelMappingName.Text = "Name:";
            // 
            // InputMappingForm
            // 
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.AutoScroll = true;
            this.ClientSize = new System.Drawing.Size(590, 608);
            this.Controls.Add(this.buttonImport);
            this.Controls.Add(this.groupBoxMapping);
            this.Controls.Add(this.groupBoxButton);
            this.Controls.Add(this.labelExpand);
            this.Controls.Add(this.treeMapping);
            this.Controls.Add(this.buttonReset);
            this.Controls.Add(this.buttonRemove);
            this.Controls.Add(this.buttonNew);
            this.Controls.Add(this.buttonDown);
            this.Controls.Add(this.buttonUp);
            this.Controls.Add(this.beveledLine1);
            this.Controls.Add(this.buttonSave);
            this.Controls.Add(this.buttonSaveClose);
            this.Controls.Add(this.buttonCancel);
            this.Controls.Add(this.headerLabel);
            this.Controls.Add(this.groupBoxAction);
            this.Controls.Add(this.groupBoxCondition);
            this.Controls.Add(this.groupBoxLayer);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
            this.HelpButton = true;
            this.Name = "InputMappingForm";
            this.ShowInTaskbar = false;
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
            this.Text = "MediaPortal - Setup";
            this.Shown += new System.EventHandler(this.InputMappingForm_Shown);
            this.groupBoxAction.ResumeLayout(false);
            this.groupBoxAction.PerformLayout();
            this.groupBoxCondition.ResumeLayout(false);
            this.groupBoxCondition.PerformLayout();
            this.groupBoxLayer.ResumeLayout(false);
            this.groupBoxLayer.PerformLayout();
            this.groupBoxButton.ResumeLayout(false);
            this.groupBoxButton.PerformLayout();
            this.groupBoxMapping.ResumeLayout(false);
            this.groupBoxMapping.PerformLayout();
            this.ResumeLayout(false);
            this.PerformLayout();

        }
        #endregion

        void CloseThread()
        {
            Thread.Sleep(200);
            Log.Info("CloseThread called");
            //this.Close();
        }

        void InitializeTreeMapping(string inputClassName)
        {
            treeMapping.Nodes.Clear();
            rootNode = new TreeNode("Mappings");
            rootNode.Tag = new Data("MAPPINGS", null, null);
            treeMapping.Nodes.Add(rootNode);
            LoadMapping(inputClassName + ".xml", false);
            rootNode.Expand();
        }

        void LoadMapping(string xmlFile, bool defaults)
        {
            groupBoxMapping.Enabled = false;
            groupBoxButton.Enabled = false;
            groupBoxLayer.Enabled = false;
            groupBoxCondition.Enabled = false;
            groupBoxAction.Enabled = false;
            string path = Config.GetFolder(Config.Dir.Base) + "\\InputDeviceMappings\\defaults\\" + xmlFile;
            if (!defaults && File.Exists(Config.GetFile(Config.Dir.CustomInputDevice, xmlFile)))
                path = Config.GetFile(Config.Dir.CustomInputDevice, xmlFile);
                
            try
            {
                AddMapping(path);
            }
            catch (Exception ex)
            {
                Log.Error(LOG_PREFIX + ex);
                File.Delete(Config.GetFile(Config.Dir.CustomInputDevice, xmlFile));
                if ( !defaults ) { LoadMapping(xmlFile, true); }
            }
        }

        private void AddMapping(string path)
        {
            if (File.Exists(path))
            {
                XmlDocument doc = new XmlDocument();
                doc.Load(path);
                XmlNodeList listRemotes = doc.DocumentElement.SelectNodes("/mappings/remote");

                foreach (XmlNode nodeRemote in listRemotes)
                {
                    TreeNode remoteNode = new TreeNode(); // KBIP Removed parameter
                    remoteNode.Tag = new Data("REMOTE", null, nodeRemote.Attributes["family"].Value);
                    updateRemoteNode(remoteNode); // KBIP ADDED
                    XmlNodeList listButtons = nodeRemote.SelectNodes("button");
                    foreach (XmlNode nodeButton in listButtons)
                    {
                        TreeNode buttonNode = new TreeNode(); // KBIP Removed parameter
                        buttonNode.Tag = new Data("BUTTON", nodeButton.Attributes["name"].Value, nodeButton.Attributes["code"].Value);
                        updateButtonNode(buttonNode); // KBIP ADDED
                        remoteNode.Nodes.Add(buttonNode);

                        TreeNode layer1Node = new TreeNode("Layer 1");
                        TreeNode layer2Node = new TreeNode("Layer 2");
                        TreeNode layerAllNode = new TreeNode("All Layers");
                        layer1Node.Tag = new Data("LAYER", null, "1");
                        layer2Node.Tag = new Data("LAYER", null, "2");
                        layerAllNode.Tag = new Data("LAYER", null, "0");
                        layer1Node.ForeColor = Color.DimGray;
                        layer2Node.ForeColor = Color.DimGray;
                        layerAllNode.ForeColor = Color.DimGray;

                        XmlNodeList listActions = nodeButton.SelectNodes("action");

                        foreach (XmlNode nodeAction in listActions)
                        {
                            string conditionString = string.Empty;
                            string commandString = string.Empty;

                            string condition = nodeAction.Attributes["condition"].Value.ToUpper();
                            string conProperty = nodeAction.Attributes["conproperty"].Value.ToUpper();
                            string command = nodeAction.Attributes["command"].Value.ToUpper();
                            string cmdProperty = nodeAction.Attributes["cmdproperty"].Value.ToUpper();
                            string sound = string.Empty;
                            XmlAttribute soundAttribute = nodeAction.Attributes["sound"];
                            if (soundAttribute != null)
                                sound = soundAttribute.Value;
                            bool gainFocus = false;
                            XmlAttribute focusAttribute = nodeAction.Attributes["focus"];
                            if (focusAttribute != null)
                                gainFocus = Convert.ToBoolean(focusAttribute.Value);
                            int layer = Convert.ToInt32(nodeAction.Attributes["layer"].Value);

                            #region Conditions

                            switch (condition)
                            {
                                case "WINDOW":
                                    // KBIP: Changed following line: Enum retrieval -> GetWindowName method
                                    conditionString = GetFriendlyName(GetWindowNameForId(Convert.ToInt32(conProperty)));
                                    break;
                                case "FULLSCREEN":
                                    if (conProperty == "TRUE")
                                        conditionString = "Fullscreen";
                                    else
                                        conditionString = "No Fullscreen";
                                    break;
                                case "PLAYER":
                                    conditionString = playerList[Array.IndexOf(nativePlayerList, conProperty)];
                                    break;
                                case "*":
                                    conditionString = "No Condition";
                                    break;
                            }

                            #endregion
                            #region Commands

                            switch (command)
                            {
                                case "ACTION":
                                    commandString = "Action \"" + GetFriendlyName(Enum.GetName(typeof(MediaPortal.GUI.Library.Action.ActionType), Convert.ToInt32(cmdProperty))) + "\"";
                                    break;
                                case "KEY":
                                    commandString = "Key \"" + cmdProperty + "\"";
                                    break;
                                case "WINDOW":
                                    // KBIP: Changed following line: Enum retrieval -> GetWindowName method
                                    commandString = "Window \"" + GetFriendlyName(GetWindowNameForId(Convert.ToInt32(cmdProperty))) + "\"";
                                    break;
                                case "TOGGLE":
                                    commandString = "Toggle Layer";
                                    break;
                                case "POWER":
                                    commandString = powerList[Array.IndexOf(nativePowerList, cmdProperty)];
                                    break;
                                case "PROCESS":
                                    commandString = processList[Array.IndexOf(nativeProcessList, cmdProperty)];
                                    break;
                            }

                            #endregion

                            TreeNode conditionNode = new TreeNode(conditionString);
                            conditionNode.Tag = new Data("CONDITION", condition, conProperty);
                            if (commandString == "Action \"Key Pressed\"")
                            {
                                string cmdKeyChar = nodeAction.Attributes["cmdkeychar"].Value;
                                string cmdKeyCode = nodeAction.Attributes["cmdkeycode"].Value;
                                TreeNode commandNode = new TreeNode(string.Format("Key Pressed: {0} [{1}]", cmdKeyChar, cmdKeyCode));

                                Key key = new Key(Convert.ToInt32(cmdKeyChar), Convert.ToInt32(cmdKeyCode));

                                commandNode.Tag = new Data("COMMAND", "KEY", key, gainFocus);
                                commandNode.ForeColor = Color.DarkGreen;
                                conditionNode.ForeColor = Color.Blue;
                                conditionNode.Nodes.Add(commandNode);
                            }
                            else
                            {
                                TreeNode commandNode = new TreeNode(commandString);
                                commandNode.Tag = new Data("COMMAND", command, cmdProperty, gainFocus);
                                commandNode.ForeColor = Color.DarkGreen;
                                conditionNode.ForeColor = Color.Blue;
                                conditionNode.Nodes.Add(commandNode);
                            }

                            TreeNode soundNode = new TreeNode(sound);
                            soundNode.Tag = new Data("SOUND", null, sound);
                            if (sound == string.Empty)
                                soundNode.Text = "No Sound";
                            soundNode.ForeColor = Color.DarkRed;
                            conditionNode.Nodes.Add(soundNode);

                            if (layer == 1) layer1Node.Nodes.Add(conditionNode);
                            if (layer == 2) layer2Node.Nodes.Add(conditionNode);
                            if (layer == 0) layerAllNode.Nodes.Add(conditionNode);
                        }
                        if (layer1Node.Nodes.Count > 0) buttonNode.Nodes.Add(layer1Node);
                        if (layer2Node.Nodes.Count > 0) buttonNode.Nodes.Add(layer2Node);
                        if (layerAllNode.Nodes.Count > 0) buttonNode.Nodes.Add(layerAllNode);
                    }
                    rootNode.Nodes.Add(remoteNode);
                    if (listRemotes.Count == 1)
                        remoteNode.Expand();
                }
            }
        }

        bool SaveMapping(string xmlFile)
        {
#if !DEBUG
      try
#endif
            {
                DirectoryInfo dir = Directory.CreateDirectory(Config.GetFolder(Config.Dir.CustomInputDevice));
            }
#if !DEBUG
      catch
      {
        Log.Info(LOG_PREFIX + "MAP: Error accessing directory \"InputDeviceMappings\\custom\"");
      }

      //try
#endif
            {
                XmlTextWriter writer = new XmlTextWriter(Config.GetFile(Config.Dir.CustomInputDevice, xmlFile), System.Text.Encoding.UTF8);
                writer.Formatting = Formatting.Indented;
                writer.Indentation = 1;
                writer.IndentChar = (char)9;
                writer.WriteStartDocument(true);
                writer.WriteStartElement("mappings"); // <mappings>
                writer.WriteAttributeString("version", "3");
                if (rootNode.Nodes.Count > 0)
                    foreach (TreeNode remoteNode in rootNode.Nodes)
                    {
                        writer.WriteStartElement("remote"); // <remote>
                        writer.WriteAttributeString("family", (string)((Data)remoteNode.Tag).Value);
                        if (remoteNode.Nodes.Count > 0)
                            foreach (TreeNode buttonNode in remoteNode.Nodes)
                            {
                                writer.WriteStartElement("button"); // <button>
                                writer.WriteAttributeString("name", (string)((Data)buttonNode.Tag).Parameter);
                                writer.WriteAttributeString("code", (string)((Data)buttonNode.Tag).Value);

                                if (buttonNode.Nodes.Count > 0)
                                    foreach (TreeNode layerNode in buttonNode.Nodes)
                                    {
                                        foreach (TreeNode conditionNode in layerNode.Nodes)
                                        {
                                            string layer;
                                            string condition;
                                            string conProperty;
                                            string command = string.Empty;
                                            string cmdProperty = string.Empty;
                                            string cmdKeyChar = string.Empty;
                                            string cmdKeyCode = string.Empty;
                                            string sound = string.Empty;
                                            bool focus = false;
                                            foreach (TreeNode commandNode in conditionNode.Nodes)
                                            {
                                                switch (((Data)commandNode.Tag).Type)
                                                {
                                                    case "COMMAND":
                                                        {
                                                            command = (string)((Data)commandNode.Tag).Parameter;
                                                            focus = ((Data)commandNode.Tag).Focus;
                                                            if (command != "KEY")
                                                                cmdProperty = ((Data)commandNode.Tag).Value.ToString();
                                                            else
                                                            {
                                                                command = "ACTION";
                                                                Key key = (Key)((Data)commandNode.Tag).Value;
                                                                cmdProperty = "93";
                                                                cmdKeyChar = key.KeyChar.ToString();
                                                                cmdKeyCode = key.KeyCode.ToString();
                                                            }
                                                        }
                                                        break;
                                                    case "SOUND":
                                                        sound = (string)((Data)commandNode.Tag).Value;
                                                        break;
                                                }
                                            }
                                            condition = (string)((Data)conditionNode.Tag).Parameter;
                                            conProperty = ((Data)conditionNode.Tag).Value.ToString();
                                            layer = Convert.ToString(((Data)layerNode.Tag).Value);
                                            writer.WriteStartElement("action"); // <action>
                                            writer.WriteAttributeString("layer", layer);
                                            writer.WriteAttributeString("condition", condition);
                                            writer.WriteAttributeString("conproperty", conProperty);
                                            writer.WriteAttributeString("command", command);
                                            writer.WriteAttributeString("cmdproperty", cmdProperty);
                                            if (cmdProperty == Convert.ToInt32(MediaPortal.GUI.Library.Action.ActionType.ACTION_KEY_PRESSED).ToString())
                                            {
                                                if (cmdKeyChar != string.Empty)
                                                {
                                                    writer.WriteAttributeString("cmdkeychar", cmdKeyChar);
                                                }
                                                else
                                                {
                                                    writer.WriteAttributeString("cmdkeychar", "0");
                                                }
                                                if (cmdKeyCode != string.Empty)
                                                {
                                                    writer.WriteAttributeString("cmdkeycode", cmdKeyCode);
                                                }
                                                else
                                                {
                                                    writer.WriteAttributeString("cmdkeychar", "0");
                                                }

                                            }
                                            if (sound != string.Empty)
                                                writer.WriteAttributeString("sound", sound);
                                            if (focus)
                                                writer.WriteAttributeString("focus", focus.ToString());
                                            writer.WriteEndElement(); // </action>
                                        }
                                    }
                                writer.WriteEndElement(); // </button>
                            }
                        writer.WriteEndElement(); // </remote>
                    }
                writer.WriteEndElement(); // </mapping>
                writer.WriteEndDocument();
                writer.Close();
                changedSettings = false;
                return true;
            }
#if !DEBUG
      //catch (Exception ex)
      //{
      //  Log.Info(LOG_PREFIX + "MAP: Error saving mapping to XML file: {0}", ex.Message);
      //  return false;
      //}
#endif
        }

        TreeNode getNode(string type)
        {
            TreeNode node = treeMapping.SelectedNode;
            Data data = (Data)node.Tag;
            if (data.Type == type)
                return node;
            #region Find Node

            switch (type)
            {
                case "COMMAND":
                    if ((data.Type == "SOUND") || (data.Type == "KEY"))
                    {
                        node = node.Parent;
                        foreach (TreeNode subNode in node.Nodes)
                        {
                            data = (Data)subNode.Tag;
                            if (data.Type == type)
                                return subNode;
                        }
                    }
                    else if (data.Type == "CONDITION")
                    {
                        foreach (TreeNode subNode in node.Nodes)
                        {
                            data = (Data)subNode.Tag;
                            if (data.Type == type)
                                return subNode;
                        }
                    }
                    break;
                case "SOUND":
                    if ((data.Type == "COMMAND") || (data.Type == "KEY"))
                    {
                        node = node.Parent;
                        foreach (TreeNode subNode in node.Nodes)
                        {
                            data = (Data)subNode.Tag;
                            if (data.Type == type)
                                return subNode;
                        }
                    }
                    else if (data.Type == "CONDITION")
                    {
                        foreach (TreeNode subNode in node.Nodes)
                        {
                            data = (Data)subNode.Tag;
                            if (data.Type == type)
                                return subNode;
                        }
                    }
                    break;
                case "CONDITION":
                    if ((data.Type == "SOUND") || (data.Type == "COMMAND") || (data.Type == "KEY"))
                        return node.Parent;
                    break;
                case "LAYER":
                    if ((data.Type == "SOUND") || (data.Type == "COMMAND") || (data.Type == "KEY"))
                        return node.Parent.Parent;
                    else if (data.Type == "CONDITION")
                        return node.Parent;
                    break;
                case "BUTTON":
                    if ((data.Type == "SOUND") || (data.Type == "COMMAND") || (data.Type == "KEY"))
                        return node.Parent.Parent.Parent;
                    else if (data.Type == "CONDITION")
                        return node.Parent.Parent;
                    else if (data.Type == "LAYER")
                        return node.Parent;
                    break;
                case "REMOTE":
                    if ((data.Type == "SOUND") || (data.Type == "COMMAND") || (data.Type == "KEY"))
                        return node.Parent.Parent.Parent.Parent;
                    else if (data.Type == "CONDITION")
                        return node.Parent.Parent.Parent;
                    else if (data.Type == "LAYER")
                        return node.Parent.Parent;
                    else if (data.Type == "BUTTON")
                        return node.Parent;
                    break;
            }

            #endregion
            return null;
        }

        private void treeMapping_AfterSelect(object sender, System.Windows.Forms.TreeViewEventArgs e)
        {
            if (e.Action == TreeViewAction.Unknown)
                return;

            TreeNode node = e.Node;
            Data data = (Data)node.Tag;
            // KBIP ADDED - START
            if (getNode("REMOTE") == null)
            {
                groupBoxMapping.Enabled = false;
                textBoxMappingName.Text = "";
            }
            else
            {
                groupBoxMapping.Enabled = true;
                Data remoteData = (Data)getNode("REMOTE").Tag;
                textBoxMappingName.Text = (string)remoteData.Value;
            }

            if (getNode("BUTTON") == null)
            {
                groupBoxButton.Enabled = false;
                textKeyName.Text = "";
                textKeyDesc.Text = "";
            }
            else
            {
                groupBoxButton.Enabled = true;
                Data buttonData = (Data)getNode("BUTTON").Tag;
                textKeyName.Text = (string)buttonData.Parameter;
                textKeyDesc.Text = KeyboardHandler.GetKeycodeDesc(Int32.Parse((string)buttonData.Value));
            }

            // KBIP ADDED - END
            switch (data.Type)
            {
                case "REMOTE":
                case "BUTTON":
                    groupBoxLayer.Enabled = false;
                    groupBoxCondition.Enabled = false;
                    groupBoxAction.Enabled = false;
                    comboBoxLayer.Text = "All Layers";
                    comboBoxCondProperty.Text = "none";
                    comboBoxCmdProperty.Text = "none";
                    comboBoxSound.Text = "none";
                    return;
                case "LAYER":
                    groupBoxButton.Enabled = true; // KBIP ADDED
                    groupBoxLayer.Enabled = true;
                    groupBoxCondition.Enabled = false;
                    groupBoxAction.Enabled = false;
                    comboBoxCondProperty.Text = "none";
                    comboBoxCmdProperty.Text = "none";
                    comboBoxSound.Text = "none";
                    comboBoxLayer.SelectedIndex = Convert.ToInt32(data.Value);
                    return;
                case "COMMAND":
                case "SOUND":
                case "KEY":
                case "CONDITION":
                    {
                        groupBoxButton.Enabled = true; // KBIP ADDED
                        groupBoxCondition.Enabled = true;
                        groupBoxAction.Enabled = true;
                        groupBoxLayer.Enabled = true;
                        if ((data.Type == "COMMAND") || (data.Type == "SOUND"))
                        {
                            comboBoxLayer.SelectedIndex = Convert.ToInt32(((Data)node.Parent.Parent.Tag).Value);
                            node = node.Parent;
                            data = (Data)node.Tag;
                        }
                        else
                            comboBoxLayer.SelectedIndex = Convert.ToInt32(((Data)node.Parent.Tag).Value);

                        switch ((string)data.Parameter)
                        {
                            case "WINDOW":
                                radioButtonWindow.Checked = true;
                                comboBoxCondProperty.Enabled = true;
                                // KBIP: Changed following line: Enum retrieval -> GetWindowName method
                                UpdateCombo(ref comboBoxCondProperty, windowsList, GetFriendlyName(GetWindowNameForId(Convert.ToInt32(data.Value))));
                                break;
                            case "FULLSCREEN":
                                radioButtonFullscreen.Checked = true;
                                comboBoxCondProperty.Enabled = true;
                                if (Convert.ToBoolean(data.Value))
                                    UpdateCombo(ref comboBoxCondProperty, fullScreenList, "Fullscreen");
                                else
                                    UpdateCombo(ref comboBoxCondProperty, fullScreenList, "No Fullscreen");
                                break;
                            case "PLAYER":
                                radioButtonPlaying.Checked = true;
                                comboBoxCondProperty.Enabled = true;
                                UpdateCombo(ref comboBoxCondProperty, playerList, playerList[Array.IndexOf(nativePlayerList, (string)data.Value)]);
                                break;
                            case "*":
                                comboBoxCondProperty.Text = "none";
                                radioButtonNoCondition.Checked = true;
                                comboBoxCondProperty.Enabled = false;
                                comboBoxCondProperty.Items.Clear();
                                break;
                        }
                        foreach (TreeNode typeNode in node.Nodes)
                        {
                            data = (Data)typeNode.Tag;
                            switch (data.Type)
                            {
                                case "SOUND":
                                    if ((string)data.Value != string.Empty)
                                        comboBoxSound.SelectedItem = data.Value;
                                    else
                                        comboBoxSound.SelectedItem = "none";
                                    break;
                                case "COMMAND":
                                    checkBoxGainFocus.Checked = data.Focus;
                                    switch ((string)data.Parameter)
                                    {
                                        case "ACTION":
                                            comboBoxCmdProperty.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
                                            radioButtonAction.Checked = true;
                                            comboBoxSound.Enabled = true;
                                            comboBoxCmdProperty.Enabled = true;
                                            textBoxKeyChar.Enabled = textBoxKeyCode.Enabled = false;
                                            textBoxKeyChar.Text = textBoxKeyCode.Text = string.Empty;
                                            UpdateCombo(ref comboBoxCmdProperty, actionList, GetFriendlyName(Enum.GetName(typeof(MediaPortal.GUI.Library.Action.ActionType), Convert.ToInt32(data.Value))));
                                            break;
                                        case "KEY":
                                            comboBoxCmdProperty.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
                                            radioButtonAction.Checked = true;
                                            textBoxKeyChar.Enabled = textBoxKeyCode.Enabled = true;
                                            textBoxKeyChar.Text = ((Key)data.Value).KeyChar.ToString();
                                            textBoxKeyCode.Text = ((Key)data.Value).KeyCode.ToString();
                                            comboBoxCmdProperty.Enabled = true;
                                            UpdateCombo(ref comboBoxCmdProperty, actionList, "Key Pressed");
                                            break;
                                        case "WINDOW":
                                            comboBoxCmdProperty.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
                                            radioButtonActWindow.Checked = true;
                                            comboBoxSound.Enabled = true;
                                            comboBoxCmdProperty.Enabled = true;
                                            textBoxKeyChar.Enabled = textBoxKeyCode.Enabled = false;
                                            textBoxKeyChar.Text = textBoxKeyCode.Text = string.Empty;
                                            // KBIP: Changed following line: Enum retrieval -> GetWindowName method
                                            UpdateCombo(ref comboBoxCmdProperty, windowsListFiltered, GetFriendlyName(GetWindowNameForId(Convert.ToInt32(data.Value))));
                                            break;
                                        case "TOGGLE":
                                            radioButtonToggle.Checked = true;
                                            comboBoxSound.Enabled = true;
                                            comboBoxCmdProperty.Enabled = false;
                                            textBoxKeyChar.Enabled = textBoxKeyCode.Enabled = false;
                                            textBoxKeyChar.Text = textBoxKeyCode.Text = string.Empty;
                                            comboBoxCmdProperty.Items.Clear();
                                            comboBoxCmdProperty.Text = string.Empty;
                                            break;
                                        case "POWER":
                                            comboBoxCmdProperty.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
                                            radioButtonPower.Checked = true;
                                            comboBoxSound.Enabled = true;
                                            comboBoxCmdProperty.Enabled = true;
                                            textBoxKeyChar.Enabled = textBoxKeyCode.Enabled = false;
                                            textBoxKeyChar.Text = textBoxKeyCode.Text = string.Empty;
                                            string friendlyName = string.Empty;
                                            UpdateCombo(ref comboBoxCmdProperty, powerList, powerList[Array.IndexOf(nativePowerList, (string)data.Value)]);
                                            break;
                                        case "PROCESS":
                                            comboBoxCmdProperty.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
                                            radioButtonProcess.Checked = true;
                                            comboBoxSound.Enabled = true;
                                            comboBoxCmdProperty.Enabled = true;
                                            textBoxKeyChar.Enabled = textBoxKeyCode.Enabled = false;
                                            textBoxKeyChar.Text = textBoxKeyCode.Text = string.Empty;
                                            UpdateCombo(ref comboBoxCmdProperty, processList, processList[Array.IndexOf(nativeProcessList, (string)data.Value)]);
                                            break;
                                    }
                                    break;
                            }
                        }
                    }
                    break;
            }
        }

        void UpdateCombo(ref MediaPortal.UserInterface.Controls.MPComboBox comboBox, Array list, string hilight)
        {
            comboBox.Items.Clear();
            foreach (object item in list)
                comboBox.Items.Add(item.ToString());
            comboBox.Text = hilight;
            comboBox.SelectedItem = hilight;
            comboBox.Enabled = true;
        }

        void UpdateCombo(ref MediaPortal.UserInterface.Controls.MPComboBox comboBox, ArrayList list, string hilight)
        {
            UpdateCombo(ref comboBox, list.ToArray(), hilight);
        }

        string GetFriendlyName(string name)
        {
            if ((name.IndexOf("ACTION") != -1) || (name.IndexOf("WINDOW") != -1))
                name = name.Substring(7);

            bool upcase = true;
            string newName = string.Empty;

            foreach (char c in name)
            {
                if (c == '_')
                {
                    newName += " ";
                    upcase = true;
                }
                else if (upcase)
                {
                    newName += c.ToString();
                    upcase = false;
                }
                else
                {
                    newName += c.ToString().ToLower();
                }
            }

            CleanAbbreviation(ref newName, "TV");
            CleanAbbreviation(ref newName, "DVD");
            CleanAbbreviation(ref newName, "UI");
            CleanAbbreviation(ref newName, "Guide");
            CleanAbbreviation(ref newName, "MSN");
            CleanAbbreviation(ref newName, "OSD");
            CleanAbbreviation(ref newName, "LCD");
            CleanAbbreviation(ref newName, "EPG");
            CleanAbbreviation(ref newName, "DVBC");
            CleanAbbreviation(ref newName, "DVBS");
            CleanAbbreviation(ref newName, "DVBT");

            return newName;
        }

        string GetWindowName(string friendlyName)
        {
            return "WINDOW_" + friendlyName.Replace(' ', '_').ToUpper();
        }

        string GetActionName(string friendlyName)
        {
            string actionName = string.Empty;

            try
            {
                if (Enum.Parse(typeof(MediaPortal.GUI.Library.Action.ActionType), "ACTION_" + friendlyName.Replace(' ', '_').ToUpper()) != null)
                    actionName = "ACTION_" + friendlyName.Replace(' ', '_').ToUpper();
            }
            catch (ArgumentException)
            {
                try
                {
                    if (Enum.Parse(typeof(MediaPortal.GUI.Library.Action.ActionType), friendlyName.Replace(' ', '_').ToUpper()) != null)
                        actionName = friendlyName.Replace(' ', '_').ToUpper();
                }
                catch (ArgumentException)
                { }
            }

            return actionName;
        }


        void CleanAbbreviation(ref string name, string abbreviation)
        {
            int index = name.ToUpper().IndexOf(abbreviation.ToUpper());
            if (index != -1)
                name = name.Substring(0, index) + abbreviation + name.Substring(index + abbreviation.Length);
        }

        private void radioButtonWindow_Click(object sender, System.EventArgs e)
        {
            comboBoxCondProperty.Enabled = true;
            TreeNode node = getNode("CONDITION");
            node.Tag = new Data("CONDITION", "WINDOW", "0");
            // KBIP: Changed following line: Enum retrieval -> GetWindowName method
            UpdateCombo(ref comboBoxCondProperty, windowsList, GetFriendlyName(GetWindowNameForId(0)));
            node.Text = (string)comboBoxCondProperty.SelectedItem;
            changedSettings = true;
        }

        private void radioButtonFullscreen_Click(object sender, System.EventArgs e)
        {
            comboBoxCondProperty.Enabled = true;
            TreeNode node = getNode("CONDITION");
            node.Tag = new Data("CONDITION", "FULLSCREEN", "true");
            UpdateCombo(ref comboBoxCondProperty, fullScreenList, "Fullscreen");
            node.Text = (string)comboBoxCondProperty.SelectedItem;
            changedSettings = true;
        }

        private void radioButtonPlaying_Click(object sender, System.EventArgs e)
        {
            comboBoxCondProperty.Enabled = true;
            TreeNode node = getNode("CONDITION");
            node.Tag = new Data("CONDITION", "PLAYER", "TV");
            node.Text = playerList[0];
            UpdateCombo(ref comboBoxCondProperty, playerList, playerList[0]);
            changedSettings = true;
        }

        private void radioButtonNoCondition_Click(object sender, System.EventArgs e)
        {
            comboBoxCondProperty.Enabled = false;
            comboBoxCondProperty.Items.Clear();
            comboBoxCondProperty.Text = "none";
            TreeNode node = getNode("CONDITION");
            node.Tag = new Data("CONDITION", "*", null);
            node.Text = "No Condition";
            changedSettings = true;
        }

        private void radioButtonAction_Click(object sender, System.EventArgs e)
        {
            textBoxKeyChar.Enabled = textBoxKeyCode.Enabled = false;
            textBoxKeyChar.Text = textBoxKeyCode.Text = string.Empty;
            comboBoxCmdProperty.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            comboBoxSound.Enabled = true;
            comboBoxCmdProperty.Enabled = true;
            TreeNode node = getNode("COMMAND");
            Data data = new Data("COMMAND", "ACTION", "7");
            node.Tag = data;
            UpdateCombo(ref comboBoxCmdProperty, actionList, GetFriendlyName(Enum.GetName(typeof(MediaPortal.GUI.Library.Action.ActionType), Convert.ToInt32(data.Value))));
            node.Text = "Action \"" + (string)comboBoxCmdProperty.SelectedItem + "\"";
            ((Data)node.Tag).Focus = checkBoxGainFocus.Checked;
            changedSettings = true;
        }

        private void radioButtonActWindow_Click(object sender, System.EventArgs e)
        {
            comboBoxCmdProperty.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            comboBoxSound.Enabled = true;
            comboBoxCmdProperty.Enabled = true;
            TreeNode node = getNode("COMMAND");
            Data data = new Data("COMMAND", "WINDOW", "0");
            node.Tag = data;
            UpdateCombo(ref comboBoxCmdProperty, windowsListFiltered, GetFriendlyName(GetWindowNameForId(Convert.ToInt32(data.Value))));
            node.Text = "Window \"" + (string)comboBoxCmdProperty.SelectedItem + "\"";
            ((Data)node.Tag).Focus = checkBoxGainFocus.Checked;
            changedSettings = true;
        }

        private void radioButtonToggle_Click(object sender, System.EventArgs e)
        {
            textBoxKeyChar.Enabled = textBoxKeyCode.Enabled = false;
            textBoxKeyChar.Text = textBoxKeyCode.Text = string.Empty;
            comboBoxSound.Enabled = true;
            comboBoxCmdProperty.Enabled = false;
            comboBoxCmdProperty.Items.Clear();
            comboBoxCmdProperty.Text = "none";
            TreeNode node = getNode("COMMAND");
            Data data = new Data("COMMAND", "TOGGLE", "-1");
            node.Tag = data;
            node.Text = "Toggle Layer";
            ((Data)node.Tag).Focus = checkBoxGainFocus.Checked;
            changedSettings = true;
        }

        private void radioButtonPower_Click(object sender, System.EventArgs e)
        {
            textBoxKeyChar.Enabled = textBoxKeyCode.Enabled = false;
            textBoxKeyChar.Text = textBoxKeyCode.Text = string.Empty;
            comboBoxCmdProperty.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            comboBoxSound.Enabled = true;
            comboBoxCmdProperty.Enabled = true;
            TreeNode node = getNode("COMMAND");
            node.Tag = new Data("COMMAND", "POWER", "EXIT");
            node.Text = powerList[0];
            UpdateCombo(ref comboBoxCmdProperty, powerList, powerList[0]);
            ((Data)node.Tag).Focus = checkBoxGainFocus.Checked;
            changedSettings = true;
        }

        private void radioButtonProcess_Click(object sender, System.EventArgs e)
        {
            textBoxKeyChar.Enabled = textBoxKeyCode.Enabled = false;
            textBoxKeyChar.Text = textBoxKeyCode.Text = string.Empty;
            comboBoxCmdProperty.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            comboBoxSound.Enabled = true;
            comboBoxCmdProperty.Enabled = true;
            TreeNode node = getNode("COMMAND");
            node.Tag = new Data("COMMAND", "PROCESS", "CLOSE");
            node.Text = processList[0];
            UpdateCombo(ref comboBoxCmdProperty, processList, processList[0]);
            ((Data)node.Tag).Focus = checkBoxGainFocus.Checked;
            changedSettings = true;
        }

        private void buttonSaveClose_Click(object sender, System.EventArgs e)
        {
		    Log.Debug(LOG_PREFIX+"SaveClose, changedSettings="+changedSettings);
            if (changedSettings)
                SaveMapping(inputClassName + ".xml");
            this.Close();
        }

        private void buttonSave_Click(object sender, System.EventArgs e)
        {
            Log.Debug(LOG_PREFIX+"Save, changedSettings="+changedSettings);
            if (changedSettings)
                SaveMapping(inputClassName + ".xml");
        }

        private void buttonUp_Click(object sender, System.EventArgs e)
        {
            bool expanded = false;
            TreeNode node = treeMapping.SelectedNode;
            if (((Data)node.Tag).Type != "BUTTON")
                expanded = node.IsExpanded;
            if ((((Data)node.Tag).Type == "COMMAND") || (((Data)node.Tag).Type == "SOUND"))
            {
                node = node.Parent;
                expanded = true;
            }
            if ((((Data)node.Tag).Type != "BUTTON") && (((Data)node.Tag).Type != "CONDITION"))
                return;
            if (node.Index > 0)
            {
                int index = node.Index - 1;
                TreeNode tmpNode = (TreeNode)node.Clone();
                TreeNode parentNode = node.Parent;
                node.Remove();
                if (expanded)
                    tmpNode.Expand();
                parentNode.Nodes.Insert(index, tmpNode);
                treeMapping.SelectedNode = tmpNode;
            }
            changedSettings = true;
        }

        private void buttonDown_Click(object sender, System.EventArgs e)
        {
            bool expanded = false;
            TreeNode node = treeMapping.SelectedNode;
            if (((Data)node.Tag).Type != "BUTTON")
                expanded = node.IsExpanded;
            if ((((Data)node.Tag).Type == "COMMAND") || (((Data)node.Tag).Type == "SOUND"))
            {
                node = node.Parent;
                expanded = true;
            }
            if ((((Data)node.Tag).Type != "BUTTON") && (((Data)node.Tag).Type != "CONDITION"))
                return;
            if (node.Index < node.Parent.Nodes.Count - 1)
            {
                int index = node.Index + 1;
                TreeNode tmpNode = (TreeNode)node.Clone();
                TreeNode parentNode = node.Parent;
                node.Remove();
                if (expanded)
                    tmpNode.Expand();
                parentNode.Nodes.Insert(index, tmpNode);
                treeMapping.SelectedNode = tmpNode;
            }
            changedSettings = true;
        }

        private void buttonRemove_Click(object sender, System.EventArgs e)
        {
            TreeNode node = treeMapping.SelectedNode;
            if (node.Equals(rootNode))
            {
                MessageBox.Show(this, "You can't remove the root node");
            } 
            else 
            {
                Data data = (Data)node.Tag;
                if ((data.Type == "COMMAND") || (data.Type == "SOUND") || (data.Type == "CONDITION"))
                {
                    node = getNode("CONDITION");
                    data = (Data)node.Tag;
                }
                DialogResult result = MessageBox.Show(this, "Are you sure you want to remove this " + data.Type.ToLower() + "?", "Remove " + data.Type.ToLower(),
                  MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);
                if (result == DialogResult.Yes)
                {
                    node.Remove();
                    changedSettings = true;
                }
            }
        }


        private void buttonNew_Click(object sender, System.EventArgs e)
        {
            TreeNode node = treeMapping.SelectedNode;
            Data data = (Data)node.Tag;

            TreeNode newLayer = new TreeNode("All Layers");
            newLayer.Tag = new Data("LAYER", null, "0");
            newLayer.ForeColor = Color.DimGray;

            TreeNode newCondition = new TreeNode("No Condition");
            newCondition.Tag = new Data("CONDITION", "*", "-1");
            newCondition.ForeColor = Color.Blue;

            TreeNode newCommand = new TreeNode("Action \"Select Item\"");
            newCommand.Tag = new Data("COMMAND", "ACTION", "7");
            newCommand.ForeColor = Color.DarkGreen;

            TreeNode newSound = new TreeNode("No Sound");
            newSound.Tag = new Data("SOUND", string.Empty, string.Empty);
            newSound.ForeColor = Color.DarkRed;

            switch (data.Type)
            {
                case "LAYER":
                    newCondition.Nodes.Add(newCommand);
                    newCondition.Nodes.Add(newSound);
                    newLayer.Nodes.Add(newCondition);
                    node.Parent.Nodes.Add(newLayer);
                    newLayer.Expand();
                    treeMapping.SelectedNode = newLayer;
                    break;
                case "CONDITION":
                    newCondition.Nodes.Add(newCommand);
                    newCondition.Nodes.Add(newSound);
                    node.Parent.Nodes.Add(newCondition);
                    newCondition.Expand();
                    treeMapping.SelectedNode = newCondition;
                    break;
                case "COMMAND":
                case "SOUND":
                    newCondition.Nodes.Add(newCommand);
                    newCondition.Nodes.Add(newSound);
                    node.Parent.Parent.Nodes.Add(newCondition);
                    newCondition.Expand();
                    treeMapping.SelectedNode = newCondition;
                    break;
                case "BUTTON":
                    newCondition.Nodes.Add(newCommand);
                    newCondition.Nodes.Add(newSound);
                    newLayer.Nodes.Add(newCondition);
                    node.Nodes.Add(newLayer);
                    newLayer.Expand();
                    treeMapping.SelectedNode = newLayer;
                    break;
                case "REMOTE":
                    // KBIP ADDED - START
                    TreeNode newButton = new TreeNode();
                    newButton.Tag = new Data("BUTTON", "New Button", "0");
                    updateButtonNode(newButton);
                    newCondition.Nodes.Add(newCommand);
                    newCondition.Nodes.Add(newSound);
                    newLayer.Nodes.Add(newCondition);
                    newButton.Nodes.Add(newLayer);
                    node.Nodes.Add(newButton);
                    node.Expand();
                    // We don't select the newButton node on purpose;
                    // this allows one to quickly add more than one button
                    // without having to re-select the root node all the time.
                    //treeMapping.SelectedNode = newButton;
                    // KBIP ADDED - END
                    break;
                case "MAPPINGS":
                    TreeNode newRemote = new TreeNode();
                    newRemote.Tag = new Data("REMOTE", null, "New Mapping");
                    updateRemoteNode(newRemote);
                    node.Nodes.Add(newRemote);
                    node.Expand();
                    treeMapping.SelectedNode = newRemote;
                    break;
                default:
                    break;
            }
            changedSettings = true;

            treeMapping_AfterSelect(this, new TreeViewEventArgs(treeMapping.SelectedNode, TreeViewAction.ByKeyboard));
        }

        private void buttonReset_Click(object sender, System.EventArgs e)
        {
            DialogResult result = MessageBox.Show(this, "Are you sure you want to reset your mappings?\nThis will remove your custom Keyboard.xml file.", "Reset Mappings",
                  MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);
            if (result == DialogResult.Yes)
            {
                if (File.Exists(Config.GetFile(Config.Dir.CustomInputDevice, inputClassName + ".xml")))
                    File.Delete(Config.GetFile(Config.Dir.CustomInputDevice, inputClassName + ".xml"));
                InitializeTreeMapping(inputClassName);
            }
        }

        private void textBoxKeyCode_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (!char.IsNumber(e.KeyChar) && e.KeyChar != 8)
            {
                e.Handled = true;
            }
        }

        private void textBoxKeyChar_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (!char.IsNumber(e.KeyChar) && e.KeyChar != 8)
            {
                e.Handled = true;
            }
        }

        private void comboBoxLayer_SelectionChangeCommitted(object sender, EventArgs e)
        {
            TreeNode node = getNode("LAYER");
            if (node != null)
            {
                node.Tag = new Data("LAYER", null, comboBoxLayer.SelectedIndex);
                if (comboBoxLayer.SelectedIndex == 0)
                    node.Text = "All Layers";
                else
                    node.Text = "Layer " + comboBoxLayer.SelectedIndex.ToString();
                changedSettings = true;
            }
        }

        private void comboBoxCondProperty_SelectionChangeCommitted(object sender, EventArgs e)
        {
            TreeNode node = getNode("CONDITION");
            if (node != null)
            {
                Data data = (Data)node.Tag;
                switch ((string)data.Parameter)
                {
                    case "WINDOW":
                        // KBIP CHANGED: Enum Parse -> GetWindowIdForName method
                        node.Tag = new Data("CONDITION", "WINDOW", GetWindowIdForName(GetWindowName((string)comboBoxCondProperty.SelectedItem)));
                        node.Text = (string)comboBoxCondProperty.SelectedItem;
                        break;
                    case "FULLSCREEN":
                        if ((string)comboBoxCondProperty.SelectedItem == "Fullscreen")
                            node.Tag = new Data("CONDITION", "FULLSCREEN", "true");
                        else
                            node.Tag = new Data("CONDITION", "FULLSCREEN", "false");
                        node.Text = (string)comboBoxCondProperty.SelectedItem;
                        break;
                    case "PLAYER":
                        {
                            node.Tag = new Data("CONDITION", "PLAYER", nativePlayerList[Array.IndexOf(playerList, (string)comboBoxCondProperty.SelectedItem)]);
                            node.Text = (string)comboBoxCondProperty.SelectedItem;
                            break;
                        }
                    case "*":
                        break;
                }
                changedSettings = true;
            }
        }

        private void comboBoxCmdProperty_SelectionChangeCommitted(object sender, EventArgs e)
        {
            if ((string)comboBoxCmdProperty.SelectedItem == "Key Pressed")
                textBoxKeyChar.Enabled = textBoxKeyCode.Enabled = true;
            else
            {
                textBoxKeyChar.Enabled = textBoxKeyCode.Enabled = false;
                textBoxKeyChar.Text = textBoxKeyCode.Text = string.Empty;
            }

            TreeNode node = getNode("COMMAND");
            if (node != null)
            {
                Data data = (Data)node.Tag;
                switch ((string)data.Parameter)
                {
                    case "ACTION":
                        if ((string)comboBoxCmdProperty.SelectedItem != "Key Pressed")
                        {
                            node.Tag = new Data("COMMAND", "ACTION", (int)Enum.Parse(typeof(MediaPortal.GUI.Library.Action.ActionType), GetActionName((string)comboBoxCmdProperty.SelectedItem)));
                            node.Text = "Action \"" + (string)comboBoxCmdProperty.SelectedItem + "\"";
                        }
                        else
                        {
                            textBoxKeyChar.Text = "0";
                            textBoxKeyCode.Text = "0";
                            Key key = new Key(Convert.ToInt32(textBoxKeyChar.Text), Convert.ToInt32(textBoxKeyCode.Text));
                            node.Tag = new Data("COMMAND", "KEY", key);
                            node.Text = string.Format("Key Pressed: {0} [{1}]", textBoxKeyChar.Text, textBoxKeyCode.Text);
                        }
                        break;
                    case "WINDOW":
                        // KBIP CHANGED: Enum Parse -> GetWindowIdForName method
                        node.Tag = new Data("COMMAND", "WINDOW", GetWindowIdForName(GetWindowName((string)comboBoxCmdProperty.SelectedItem)));
                        node.Text = "Window \"" + (string)comboBoxCmdProperty.SelectedItem + "\"";
                        break;
                    case "POWER":
                        node.Tag = new Data("COMMAND", "POWER", nativePowerList[Array.IndexOf(powerList, (string)comboBoxCmdProperty.SelectedItem)]);
                        node.Text = (string)comboBoxCmdProperty.SelectedItem;
                        break;
                    case "PROCESS":
                        node.Tag = new Data("COMMAND", "PROCESS", nativeProcessList[Array.IndexOf(processList, (string)comboBoxCmdProperty.SelectedItem)]);
                        node.Text = (string)comboBoxCmdProperty.SelectedItem;
                        break;
                }
                ((Data)node.Tag).Focus = checkBoxGainFocus.Checked;
                changedSettings = true;
            }
        }

        private void comboBoxSound_SelectionChangeCommitted(object sender, EventArgs e)
        {
            TreeNode node = getNode("SOUND");
            if (node != null)
            {
                node.Text = (string)comboBoxSound.SelectedItem;
                if (node.Text == "none")
                {
                    node.Tag = new Data("SOUND", null, string.Empty);
                    node.Text = "No Sound";
                }
                else
                    node.Tag = new Data("SOUND", null, (string)comboBoxSound.SelectedItem);
                changedSettings = true;
            }
        }

        private void textBoxKeyChar_KeyUp(object sender, KeyEventArgs e)
        {
            string keyChar = textBoxKeyChar.Text;
            string keyCode = textBoxKeyCode.Text;
            TreeNode node = getNode("COMMAND");
            if (keyChar == string.Empty)
                keyChar = "0";
            if (keyCode == string.Empty)
                keyCode = "0";
            Key key = new Key(Convert.ToInt32(keyChar), Convert.ToInt32(keyCode));
            node.Tag = new Data("COMMAND", "KEY", key);
            node.Text = string.Format("Key Pressed: {0} [{1}]", keyChar, keyCode);
            ((Data)node.Tag).Focus = checkBoxGainFocus.Checked;
            changedSettings = true;
        }

        private void textBoxKeyCode_KeyUp(object sender, KeyEventArgs e)
        {
            textBoxKeyChar_KeyUp(sender, e);
        }

        private void labelExpand_Click(object sender, EventArgs e)
        {
            if (treeMapping.SelectedNode == null)
                treeMapping.Select();
            treeMapping.SelectedNode.ExpandAll();
        }

        private void checkBoxGainFocus_CheckedChanged(object sender, EventArgs e)
        {
            TreeNode node = getNode("COMMAND");
            ((Data)node.Tag).Focus = checkBoxGainFocus.Checked;
            changedSettings = true;
        }

        // KBIP ADDED - START
        private Hashtable windowIdsToNameMap = new Hashtable();

        private void UpdateWindowsForPlugins()
        {
            string directory = Config.GetSubFolder(Config.Dir.Plugins, "windows");
            if (Directory.Exists(directory))
            {
                //
                // Enumerate files
                //
                string[] files = Directory.GetFiles(directory, "*.dll");

                //
                // Add to list
                //
                foreach (string file in files)
                {
                    UpdateWindowsForPlugin(file);
                }
            }
        }

        private void UpdateWindowsForPlugin(string pluginFile)
        {
            Assembly pluginAssembly = null;
            try
            {
                Log.Debug(LOG_PREFIX + "UpdateWindowsForPlugin: loadPlugins {0}", pluginFile);
                pluginAssembly = Assembly.LoadFrom(pluginFile);
            }
            catch (BadImageFormatException)
            {
                Log.Warn(LOG_PREFIX + "PluginsNew: {0} has a bad image format", pluginFile);
            }

            if (pluginAssembly != null)
            {
                try
                {
                    Type[] exportedTypes = pluginAssembly.GetExportedTypes();

                    foreach (Type type in exportedTypes)
                    {
                        if ((!type.IsAbstract) && (type.IsClass) && (type.IsSubclassOf(typeof(GUIWindow))))
                        {
                            object newObj;
                            try
                            {
                                newObj = Activator.CreateInstance(type);
                            }
                            catch (TargetInvocationException e)
                            {
                                Log.Warn(LOG_PREFIX + "Error loading window {0} from plugin {1}: {2}", new object[] { type, pluginFile, e });
                                continue;
                            }
                            GUIWindow win = (GUIWindow)newObj;
                            int id = win.GetID;
                            if (!windowIdsToNameMap.ContainsKey(id) && !Enum.IsDefined(typeof(GUIWindow.Window), id))
                            {
                                string name = "WINDOW_" + "[" + id + "]_" + convertTypeToWindowName(win.GetType().Name);
                                windowIdsToNameMap.Add(id, name);
                                string friendlyName = GetFriendlyName(name);
                                if (!windowsList.Contains(friendlyName)) { windowsList.Add(friendlyName); }
                                if (!windowsListFiltered.Contains(friendlyName)) { windowsListFiltered.Add(friendlyName); }
                            }
                        }
                    }
                }
                catch (Exception e)
                {
                    Log.Warn(LOG_PREFIX + "Error loading windows from plugin {0}: {1}", new object[] { pluginFile, e });
                }
            }
        }

        private static string convertTypeToWindowName(string name)
        {
            string newName = string.Empty;
            bool lastWasUpper = false;
            bool isFirstChar = true;
            foreach (char c in name)
            {
                if (Char.IsUpper(c) && !lastWasUpper && !isFirstChar)
                {
                    newName += "_" + c;
                }
                else
                {
                    newName += Char.ToUpper(c);
                }
                lastWasUpper = Char.IsUpper(c);
                isFirstChar = false;
            }
            return newName;
        }

        private string GetWindowNameForId(int windowId)
        {
            string result = (string)windowIdsToNameMap[windowId];
            if (result == null)
            {
                result = "WINDOW_[" + windowId + "]";
                windowIdsToNameMap.Add(windowId, result);
                string friendlyName = GetFriendlyName(result);
                if (!windowsList.Contains(friendlyName)) { windowsList.Add(friendlyName); }
                if (!windowsListFiltered.Contains(friendlyName)) { windowsListFiltered.Add(friendlyName); }
            }
            return result;
        }

        private int GetWindowIdForName(string windowName)
        {
            Log.Debug(LOG_PREFIX + "GetWindowIdForName, windowName=" + windowName);
            foreach (DictionaryEntry de in windowIdsToNameMap)
            {
                Log.Debug(LOG_PREFIX + "GetWindowIdForName, de.Value=" + de.Value + ", de.Key=" + de.Key);
                if (windowName.Equals(de.Value))
                {
                    return (int)de.Key;
                }
            }
            return -1;
        }

        private void updateButtonNode(TreeNode node)
        {
            Data data = (Data)node.Tag;
            node.Text = data.Parameter + " (" + KeyboardHandler.GetKeycodeDesc(Int32.Parse((string)data.Value)) + ")";
        }

        private void textKeyName_KeyUp(object sender, KeyEventArgs e)
        {
            TreeNode node = getNode("BUTTON");
            ((Data)node.Tag).Parameter = textKeyName.Text;
            updateButtonNode(node);
            changedSettings = true;
        }

        private void updateRemoteNode(TreeNode node)
        {
            Data data = (Data)node.Tag;
            node.Text = (string)data.Value;
        }

        private void textBoxMappingName_KeyUp(object sender, KeyEventArgs e)
        {
            TreeNode node = getNode("REMOTE");
            ((Data)node.Tag).Value = textBoxMappingName.Text;
            updateRemoteNode(node);
            changedSettings = true;
        }

        private void buttonImport_Click(object sender, EventArgs e)
        {
            try
            {
                OpenFileDialog dlg = new OpenFileDialog();
                dlg.Filter = "xml-file (*.xml)|*.xml";
                dlg.InitialDirectory = Config.GetFolder(Config.Dir.Base) + "\\InputDeviceMappings\\defaults\\KeyboardInputPlugin";
                if (dlg.ShowDialog() == DialogResult.OK)
                {
                    AddMapping(dlg.FileName);
                    rootNode.Expand();
                    changedSettings = true;
                }
            }
            catch (Exception exc)
            {
                MessageBox.Show(this, "Error while importing mapping: "+exc.Message);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            if (!changedSettings || 
                DialogResult.Yes == MessageBox.Show(this, 
                    "You have unsaved changed. Are you sure you want\nto close this window without saving?", "Discard Changes?",
                    MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) )
            {
                this.Close();        
            }
        }

        void ShowHelp()
        {
            if (rootNode.Nodes.Count == 0)
            {
                MessageBox.Show(this,
                    "Welcome to KeyboardInputPlugin configuration. As you have no mappings\n"+
                    "defined yet, here's a short instruction.\n\n"+
                    "The easiest way to start is by clicking the 'Import' button to \n"+
                    "import a predefined mapping. From there on, you can modify key \n" +
                    "bindings and action mappings by selecting the correct tree node \n" +
                    "and modifying its properties using the right hand side of the window.\n\n" +
                    "New mappings, buttons, layers and conditions can be added by selecting\n" +
                    "the correct parent node and then clicking the 'New' button. Existing\n" +
                    "nodes can be removed by clicking the 'Remove' button. Nodes can be re-\n" +
                    "ordered by using the 'Up' and 'Down' buttons.\n\n"+
                    "After making your modifications, you can use one of the 'Save' buttons\n" +
                    "to save your configuration. If you want to discard any changes you've \n" +
                    "made so far, click the 'Cancel' button. Using the 'Reset' button, you \n" +
                    "can reset all your modifications since installation of the plug-in, and \n" +
                    "start from scratch."
                );
            }
        }



        #region KeyboardHandler definition and delegates
        KeyboardHandler keyboardHandler = null;

        private bool KeyboardHandler_KeyPressHandler(int keycode)
        {
            if (textKeyDesc.Focused)
            {
                textKeyDesc.Text = KeyboardHandler.GetKeycodeDesc(keycode);
                TreeNode node = getNode("BUTTON");
                ((Data)node.Tag).Value = keycode.ToString();
                updateButtonNode(node);
                changedSettings = true;
                return true;
            }
            return false;
        }
        #endregion

        private void textKeyDesc_GotFocus(object sender, EventArgs e)
        {
            if (keyboardHandler == null) { 
                keyboardHandler = new KeyboardHandler(new KeyPressHandler(KeyboardHandler_KeyPressHandler)); 
            }
            keyboardHandler.IsEnabled = true;
        }

        private void textKeyDesc_LostFocus(object sender, EventArgs e)
        {
            if (keyboardHandler != null) { keyboardHandler.IsEnabled = false; }
        }

        private void InputMappingForm_Shown(object sender, EventArgs e)
        {
            ShowHelp();
        }

    // KBIP ADDED - END
    }
}
