using System;
using System.Windows.Forms;
using System.Collections;
using System.Runtime.InteropServices;
using MediaPortal.GUI.Library;
using Markos.IO.WindowsHooks;
using System.Text;

namespace KeyboardInputPlugin {

	public delegate bool KeyPressHandler(int keycode);

	public class KeyboardHandler {

		private const string LOG_PREFIX = "KeyboardInputPlugin: ";

		private bool _isEnabled = false;
		public bool IsEnabled {
			get {
				return _isEnabled;
			}

			set {
				Log.Debug(LOG_PREFIX + "IsEnabled: " + value);

				lock(this) {
					if(value) {
						if(!_isEnabled)
							Start();
					} else {
						if(_isEnabled)
							Stop();
					}
					_isEnabled = value;
				}
			}
		}

		private KeyPressHandler keyPressHandler = null;

		public KeyboardHandler(KeyPressHandler keyPressHandler) {
			this.keyPressHandler = keyPressHandler;
		}

		public static string GetKeycodeDesc(int keycode) {
			var sb = new StringBuilder(30);

			if(keycode >= 1000000) {
				sb.Append("Win+");
				keycode -= 1000000;
			}

			if(keycode >= 100000) {
				sb.Append("Alt+");
				keycode -= 100000;
			}

			if(keycode >= 10000) {
				sb.Append("Ctrl+");
				keycode -= 10000;
			}

			if(keycode >= 1000) {
				sb.Append("Shift+");
				keycode -= 1000;
			}

			try {
				sb.Append(Enum.GetName(typeof(Keys), keycode));
			} catch(ArgumentException) {
				sb.Append("Unknown");
			}

			return sb.ToString();
		}

		private void OnKeyStateChanged(object sender, KeyStateChangedEventArgs e) {
			if(!e.KeyPushed || e.IsModifier)
				return;

			int keyValue = (int)e.VKCode;
			if(e.IsShiftDown) keyValue += 1000;
			if(e.IsCtrlDown) keyValue += 10000;
			if(e.IsAltDown) keyValue += 100000;
			if(e.IsWinDown) keyValue += 1000000;

			// Handle the key press
			e.BlockEvent = keyPressHandler(keyValue);

			//this should only be called if logging enabled, otherwise causes slowdowns
			//however Log class does not contain IsDebugEnabled-like property so I am
			//disabling this completely
			//Log.Debug(LOG_PREFIX + "OnKeyDown: " + GetKeycodeDesc(keyValue) + " (" + keyValue + "), Handled: " + e.BlockEvent);
		}

		public void Start() {
			KeyboardHook.Instance.KeyStateChanged += OnKeyStateChanged;
		}

		public void Stop() {
			KeyboardHook.Instance.KeyStateChanged -= OnKeyStateChanged;
		}
	}
}
