using System;
using MediaPortal.Configuration;
using MediaPortal.GUI.Library;

namespace KeyboardInputPlugin
{
    [PluginIcons("KeyboardInputPlugin.Keyboard.jpg", "KeyboardInputPlugin.KeyboardDisabled.jpg")]
    public class KeyboardInputPluginMain : ISetupForm, IPlugin, IPluginReceiver
    {
        private const string LOG_PREFIX = "KeyboardInputPlugin: ";
        private const string MAPPING_NAME = "Keyboard";
        private const int WM_KEYDOWN = 0x0100;
        private const int WM_SYSKEYDOWN = 0x0104;
        private InputHandler _inputHandler = null;
        private KeyboardHandler keyboardHandler = null;
        private DateTime lastWndProcKeyDownTime = DateTime.Now;

        #region ISetupForm Members
        public string Author()
        {
            return ("Ruud Senden | SilentException");
        }

        public bool CanEnable()
        {
            return true;
        }

        public bool DefaultEnabled()
        {
            return true;
        }

        public string Description()
        {
            return "Keyboard Input Plugin";
        }

        public bool GetHome(out string strButtonText, out string strButtonImage, out string strButtonImageFocus, out string strPictureImage)
        {
            strButtonText = null;
            strButtonImage = null;
            strButtonImageFocus = null;
            strPictureImage = null;
            return false;
        }

        public int GetWindowId()
        {
            return 90012;
        }

        public bool HasSetup()
        {
            return true;
        }

        public string PluginName()
        {
            return "Keyboard Input Plugin";
        }

        public void ShowPlugin()
        {
            InputMappingForm dlg = new InputMappingForm(MAPPING_NAME);
            dlg.ShowDialog();
        }
        #endregion

        #region IPlugin Members
        public void Start()
        {
            _inputHandler = new InputHandler(MAPPING_NAME);
            if (!_inputHandler.IsLoaded)
            {
                Log.Error(LOG_PREFIX + "No mapping could be loaded; disabling KeyboardInputPlugin");
            }
            else
            {
                keyboardHandler = new KeyboardHandler(new KeyPressHandler(KeyboardHandler_KeyPressHandler));
                keyboardHandler.IsEnabled = true;
                Log.Info(LOG_PREFIX + "Started");
                if (GUIGraphicsContext.form != null)
                {
                    GUIGraphicsContext.form.Activated += new EventHandler(form_Activated);
                    GUIGraphicsContext.form.Deactivate += new EventHandler(form_Deactivate);
                }
            }
        }

        public void Stop()
        {
            if (keyboardHandler != null)
            {
                keyboardHandler.Stop();
                keyboardHandler = null;
            }
            Log.Info(LOG_PREFIX + "Stopped");
        }
        #endregion

        #region IPluginReceiver Members

		
        public bool WndProc(ref System.Windows.Forms.Message msg)
        {
			/*if (keyboardHandler != null && (msg.Msg == WM_KEYDOWN || msg.Msg == WM_SYSKEYDOWN))
			{
				if ( keyboardHandler.IsEnabled && keyboardHandler.LastHookInvokedTime < this.lastWndProcKeyDownTime ) {
					Log.Info(LOG_PREFIX + "KeyboardHandler seems to be inactive; restarting");
					keyboardHandler.IsEnabled = false;
					keyboardHandler.IsEnabled = true;
				}
				lastWndProcKeyDownTime = DateTime.Now;
			}*/
            return false;
        }
		

        #endregion


        #region KeyboardHandler delegates
        private bool KeyboardHandler_KeyPressHandler(int keycode)
        {
            try
            {
                return _inputHandler.MapAction(keycode);
            }
            catch (ApplicationException ae)
            {
                Log.Warn(LOG_PREFIX + "Exception while mapping keycode {0}: {1}", keycode, ae);
            }
            return false;
        }
        #endregion

        #region Events
        void form_Deactivate(object sender, EventArgs e)
        {
            keyboardHandler.IsEnabled = false;
        }

        void form_Activated(object sender, EventArgs e)
        {
            keyboardHandler.IsEnabled = true;
        }
        #endregion
    }
}
