﻿using System;

namespace Markos.IO.WindowsHooks {

	/// <summary>Class containing event data for keyboard hook.</summary>
	public class KeyStateChangedEventArgs : EventArgs {

		/// <summary>
		/// If set this to true, the event will be suppressed (and not sent to other applications).
		/// </summary>
		/// <value>
		///   <c>true</c> if block event requested; otherwise, <c>false</c>.
		/// </value>
		public bool BlockEvent { get; set; }

		/// <summary>
		/// Gets a value indicating whether Alt modifier was pressed during this event.
		/// </summary>
		/// <value>
		/// 	<c>true</c> if the Alt was down; otherwise, <c>false</c>.
		/// </value>
		public bool IsAltDown { get; private set; }

		/// <summary>
		/// Gets a value indicating whether Ctrl modifier was pressed during this event.
		/// </summary>
		/// <value>
		/// 	<c>true</c> if the Ctrl was down; otherwise, <c>false</c>.
		/// </value>
		public bool IsCtrlDown { get; private set; }

		/// <summary>
		/// Gets a value indicating whether key in this instance is modifier key.
		/// </summary>
		/// <value>
		/// 	<c>true</c> if the key is modifier key; otherwise, <c>false</c>.
		/// </value>
		public bool IsModifier { get; private set; }

		/// <summary>
		/// Gets a value indicating whether Shift modifier was pressed during this event.
		/// </summary>
		/// <value>
		/// 	<c>true</c> if the Shift was down; otherwise, <c>false</c>.
		/// </value>
		public bool IsShiftDown { get; private set; }

		/// <summary>
		/// Gets a value indicating whether Win modifier was pressed during this event.
		/// </summary>
		/// <value>
		/// 	<c>true</c> if the Win was down; otherwise, <c>false</c>.
		/// </value>
		public bool IsWinDown { get; private set; }

		/// <summary>
		/// Gets a value indicating whether key was pushed or released.
		/// </summary>
		/// <value>
		///   <c>true</c> if the key was pushed; otherwise, <c>false</c>.
		/// </value>
		public bool KeyPushed { get; private set; }

		/// <summary>
		/// Gets the virtual key code of key in this event.
		/// </summary>
		public uint VKCode { get; private set; }

		/// <summary>
		/// Initializes a new instance of the KeyEventEventArgs class.
		/// </summary>
		public KeyStateChangedEventArgs(bool isAltDown, bool isCtrlDown, bool isShiftDown, bool isWinDown, bool keyPushed, uint vkCode, bool isModifier) {
			IsAltDown = isAltDown;
			IsCtrlDown = isCtrlDown;
			IsShiftDown = isShiftDown;
			IsWinDown = isWinDown;
			KeyPushed = keyPushed;
			VKCode = vkCode;
			IsModifier = isModifier;
		}
	}
}
