using System;

namespace Markos.IO.WindowsHooks {

	/// <summary>Class containing event data for Windows hooks.</summary>
	public class WindowsHookEventArgs : EventArgs {

		/// <summary>A code that the hook procedure uses to determine how to process the message.</summary>
		public int Code { get; private set; }

		/// <summary>Gets the lParam pointer sent by Windows.</summary>
		public IntPtr LParam { get; private set; }

		/// <summary>To override value returned to windows, set this property.</summary>
		/// <value>The value returned to windows.</value>
		public IntPtr? Return { get; set; }

		/// <summary>Gets the wParam pointer sent by Windows.</summary>
		public IntPtr WParam { get; private set; }

		/// <summary>
		/// Initializes a new instance of the WindowsHookEventArgs class.
		/// </summary>
		public WindowsHookEventArgs(int code, IntPtr wParam, IntPtr lParam) {
			Code = code;
			LParam = lParam;
			WParam = wParam;
		}
	}
}
